/*
 * ActiveGroupList.java
 *
 * Created on 03 September 2008, 15:38
 *
 * encapsulates the Active Group List: the list of groups active in the game.
 *
 *@author maya
 */

package GUIComponents;

//java packages
import java.util.Random;

public class ActiveGroupsList {
    private ActiveGroupsListNode[] activeGroups;
    private boolean ordered;
    private int currentNoGroups;
    int[] deletedGroupsThisRound=null;
    
    /** Creates a new instance of activeGroupsList */
    public ActiveGroupsList(int maxNoGroups) {
        this.activeGroups = new ActiveGroupsListNode[maxNoGroups];
        ordered = false;
        currentNoGroups = 0;        
    }
    
    //setters & getters
    protected ActiveGroupsListNode getActiveGroup(int groupIndex){
        if ((groupIndex<0)||(groupIndex>=this.activeGroups.length))
            return null;
        else
            return this.activeGroups[groupIndex];
    }            
    protected ActiveGroupsListNode getFirstGroup(){
        if(this.activeGroups !=null)
            return this.activeGroups[0];
        else
            return null;
    }
    protected int getCurrentNoGroups(){return this.currentNoGroups;}
    protected short getGroupGoal(int groupIndx){
        return this.activeGroups[groupIndx-1].getGroup().getGoal();
    }
    
    //methods
    //order
    protected void order(){
        if(currentNoGroups>1){
            ActiveGroupsListNode[] tempGroups = new ActiveGroupsListNode[this.currentNoGroups];
            for(int i=0; i<this.currentNoGroups; i++){
                tempGroups[i] = this.activeGroups[i];}
                
            int groupsLeft = this.currentNoGroups;
            Random randomShuffler = new Random();
            int k;
            ActiveGroupsListNode temp;
            while (groupsLeft>0){                  
                k = randomShuffler.nextInt(groupsLeft);
                temp = tempGroups[groupsLeft-1]; 
                tempGroups[groupsLeft-1] = tempGroups[k]; 
                tempGroups[k] = temp;
                groupsLeft--;
            }
            //playersArray
            for(int i=0; i<this.currentNoGroups; i++){
                this.activeGroups[i] = tempGroups[i];
                this.activeGroups[i].setGroupIndx(i+1);
            }
            ordered = true;
        }
    }
    protected void setGroupsIndex(){
        if(this.activeGroups==null) return;
        for(int i=0; i<this.activeGroups.length; i++)
            this.activeGroups[i].setGroupIndx(i+1);
    }
    
    //add - delete groups
    protected boolean addNewGroup(Group newGroup){
        boolean done = true;
        if(this.currentNoGroups ==this.activeGroups.length)
            return false;
        else{
            ActiveGroupsListNode newActiveGroup = new ActiveGroupsListNode(newGroup);
            this.currentNoGroups++;
            this.activeGroups[currentNoGroups-1] = newActiveGroup;
            return true;
        }
    }
    protected boolean deleteGroup(int groupIndx){
        if((groupIndx<=0)||(groupIndx>this.activeGroups.length))
            return false;
        else
        {
            this.activeGroups[groupIndx]=null;
            return true;
        }
    }
    
    //initialize
    protected void initialize(short[] currentCase){
        if(this.activeGroups!=null){
            for(int i=0; i<this.activeGroups.length; i++){
                this.activeGroups[i].initialize(currentCase);
            }
        }
    }
    protected void setGroupPlayers(){
        for(int i=0; i<this.activeGroups.length; i++){
            this.activeGroups[i].setPlayers();
        }
    }
    //play game
    protected Move firstRound(){
        if(this.activeGroups!=null)            
            return this.activeGroups[0].firstRound();
        else
            return null;
    }         
    protected Move[] nextRound(int round, TMUInterface tmuInterface, GMUInterface gmuInterface, short gameFlag){        
        Move[] roundMoves = new Move[this.activeGroups.length];
        for(int i=0; i<this.activeGroups.length; i++)
        {
            if(this.activeGroups[i]!=null){
                roundMoves[i]= this.activeGroups[i].nextRound(round, tmuInterface, gmuInterface, gameFlag);
            }                
        }
        int returnRoundMovesNo =0;
        for(int i=0; i<roundMoves.length; i++){
            if(roundMoves[i]!=null)
                returnRoundMovesNo++;
        }
        if(returnRoundMovesNo==0)return null;
        else{           
           Move[] returnRoundMoves = new Move[returnRoundMovesNo];
           int j=0;
           int i=0;
           while((j<returnRoundMovesNo)&&(i<roundMoves.length))
           {
              if(roundMoves[i]!=null){
                returnRoundMoves[j] = roundMoves[i];
                j++;
              }
              i++;           
           }
           return returnRoundMoves;
        }
    } 
    
    //delete groups
    protected int[] deletedGroupsThisRound(){
        return this.deletedGroupsThisRound;
    }    
    protected boolean deleteGroups(){
        int deletedGroupsNo =0;
        for(int i=0; i<this.activeGroups.length; i++){
            if(this.activeGroups[i].getRedZoneFlag()>=2)   deletedGroupsNo++;            
        }
        if (deletedGroupsNo>0){
            this.deletedGroupsThisRound = new int[deletedGroupsNo];
            for(int i=0; i<this.activeGroups.length; i++){
                int j=0;
                if(this.activeGroups[i].getRedZoneFlag()>=2){                    
                    this.deletedGroupsThisRound()[j] = this.activeGroups[i].getGroup().getIndex();
                    this.deleteGroup(this.activeGroups[i].getGroup().getIndex());
                }
            }
            this.currentNoGroups -=deletedGroupsNo;
            return true;
        }
        else
            return false;
    }    
    protected void resetDeletedGroupsArray(){this.deletedGroupsThisRound=null;}    
    
    //output
    protected String output(int index){
        return this.activeGroups[index-1].outPut();
    }
    protected InGroupMove[][] inGroupMoves(int groupIndx){return this.activeGroups[groupIndx].inGroupMoves();}
}

