/*
 * Chairperson.java
 *
 * Created on 03 September 2008, 16:22
 *
 * Chairperson class is reponsilbe of managing PISA groups games
 *
 *@author maya
 */

package GUIComponents;

//java packages
import java.util.*;

public class Chairperson {
    private GameSettings gameSettings;//the basic settings of the game
    private short[] currentCase;//the case under discussion
    private short[] classifications;//the case possible classification
    private short gameStatus; //0=ok, 1=danger, 2=termination
    private TMU tmu;//Tree Management Unit
    private DMU dmu;//Dialogue Management Unit
    private GMU gmu;//Groups Management Unit
    private int focusingLevel=0;//1=focused, 2=strictly focused, 3=controlled strictly focused
    /*special array for gui application stores group moves according to the rounds they were played in.
     the rows are rounds and coulmns are moves per groups*/
    private Move[][] guiMovesOutput= null;
    
    //Constructors
    protected Chairperson(){
        this.gameSettings = new GameSettings();
        currentCase = null;        
        gmu = new GMU(this.gameSettings.getMaxNoGroups());
        dmu = new DMU(gameSettings.getMaxNoRounds());
    }
    protected Chairperson(int newFocusingLevel){
        this.gameSettings = new GameSettings();
        currentCase = null;        
        gmu = new GMU(this.gameSettings.getMaxNoGroups());
        dmu = new DMU(gameSettings.getMaxNoRounds());
        this.focusingLevel=newFocusingLevel;
    }
    
    //methods
    //getters
    protected Move[][] getGUIMovesOutput(){return this.guiMovesOutput;}
    protected InGroupMove[][] inGroupMoves(int groupIndx){return this.gmu.inGroupMoves(groupIndx);} 
    protected short getGroupGoal(int groupIndx){
        return this.gmu.getGroupGoal(groupIndx);
    }
    //EMBEDED DMU
    //set parameters
    protected void setGameParameters(int maxTreeSize, int maxNoRounds, int maxNoGroups, int maxNoSkips){
      this.gameSettings.setMaxNoGroups(maxNoGroups);
      this.gameSettings.setMaxNoRounds(maxNoRounds);
      this.gameSettings.setMaxNoSkips(maxNoSkips);
      this.gameSettings.setMaxTreeSize(maxTreeSize);
      this.dmu.setMaxNoRounds(maxNoRounds);
    }  
    /**returns the four games parameter such that:
     [0] = maxTreeSize, [1]maxNoRounds [2]maxNoGroups [3]maxNoSkips*/
    protected int[] getGameParameters(){return this.gameSettings.getGameParameters();}
    protected int[] getDefualtParameters(){return this.gameSettings.getDefaultParameters();}
    //set tree parameters    
    protected void setTreeParameters(double maxConfidence, double minConfidence){
       this.gameSettings.setMaxConf(maxConfidence);
       this.gameSettings.setMinConf(minConfidence);
    }    
    //set classifcations
    protected void setClassifications(short[] newClassifications){
        this.classifications = newClassifications;
        this.gmu.setClassifications(newClassifications);
    }
    protected short[] getCounterGoals(short goal){
       short[] counterGoals = new short[this.classifications.length-1];
       boolean found = false;
       int j=0;
       for(int i=0; i<classifications.length-1; i++){
           if(this.classifications[i] == goal)
               found = true;
           else{
               counterGoals[j] = this.classifications[i];
               j++;
           }
       }
       if(this.classifications[this.classifications.length-1] == goal)  found = true;       
       else{
           if(found) 
            {counterGoals[counterGoals.length-1] = this.classifications[this.classifications.length-1];}
       }
       
       if(found){return counterGoals;}
       else {return null;}        
    }  
    //set current case
    protected void setCurrentCase(short[] newCurrentCase){
        this.currentCase = newCurrentCase;
        this.gmu.setCurrentCase(newCurrentCase);
    }             
    protected void addNewGroup(short goal, GroupPlayer[] players){
        this.gmu.addNewGroup(goal,players);
    }
    protected void addNewGroup(Group group){
        this.gmu.addNewGroup(group.getGoal(),group.getPlayers());
    }
   
    //Initialize Game Settings
    protected void initializeGame(short[] newClassifications, short[] newCurrentCase){
      this.setClassifications(newClassifications);
      this.setCurrentCase(newCurrentCase);
      //Initialize Stage
      int currentPlayers = this.gmu.initialize();
      this.gameSettings.setCurrentNoPlayers(currentPlayers);
      this.dmu.initialize();
      this.tmu = new TMU(gameSettings.getCurrentNoPlayers(), gameSettings.getMaxTreeSize());
    }    
    
    //Play Game
    protected void startGame(){
        gmu.startGame();
    }
    protected void playGame(){
        this.startGame();
        boolean roundOk=this.playFirstRound();
        if(!roundOk){
            System.out.print("GAME FAILED");
            return;
        }
        while(roundOk){
            if(this.dmu.getRoundCounter()==11){
                int x=0;
            }
            roundOk=this.playNextRoundFull();
        }
        this.announceWinner();
    }   
        
    private boolean playFirstRound(){
        boolean roundsOk = this.dmu.startNewRound();
        if(roundsOk){
            Move firstMove = this.gmu.firstRound();
            this.guiMovesOutput = BasicOperations.append(this.guiMovesOutput, firstMove);
            if(firstMove!=null){
                return this.tmu.addFirstMove(firstMove);
            } 
            else
                return false;
        }
        else
            return false;
    }
    private boolean playNextRoundFull(){
        switch (this.focusingLevel){
            /*case 1: return this.playeNextRoundFocused();
            case 2: return this.playeNextRoundSFocused();*/
            case 3: return this.playNextRoundSCFocused();
            default: return this.playNextRound();
        }
    }
    
    //FOCUSING LEVELS
    private boolean playNextRound(){
      boolean roundsOk = this.dmu.startNewRound();
      if(roundsOk){
          int round = this.dmu.getRoundCounter();
          System.out.println("ROUND - " + round);
          TMUInterface tmuInterface = tmu.createInterface(this.focusingLevel);
          Move[] roundMoves = this.gmu.nextRound(round, tmuInterface, this.gameStatus);
          this.guiMovesOutput = BasicOperations.append(this.guiMovesOutput, roundMoves);
          if(roundMoves!=null){
              /*int[] deletedPlayers = this.pmu.deletedPlayersThisRound();
              if(deletedPlayers!=null){
                  this.tmu.prunePlayers(deletedPlayers);
                  this.pmu.resetDeletedPlayersArray();
              }*/
              return this.tmu.addRoundMoves(roundMoves,round);
          }
          else{
              if(this.gameStatus==0){
                  this.gameStatus=1;
                  /*int[] deletedPlayers = this.pmu.deletedPlayersThisRound();
                  if(deletedPlayers!=null){
                      this.tmu.prunePlayers(deletedPlayers);
                      this.pmu.resetDeletedPlayersArray();
                  }*/
                  return true;
              }
              else
                  return false;
          }              
          
      }
      else
          return false;
    }
    private boolean playNextRoundSCFocused(){
      boolean roundsOk = this.dmu.startNewRound();
      if(roundsOk){
          int round = this.dmu.getRoundCounter();
          System.out.println("ROUND - " + round);
          TMUInterface tmuInterface = tmu.createInterface(this.focusingLevel);
          Move[] roundMoves = this.gmu.nextRound(round, tmuInterface, this.gameStatus);
          if(roundMoves!=null){
              return this.tmu.addRoundBestMove(roundMoves,round);
          }
          else{
              if(this.gameStatus==0){
                  this.gameStatus=1;
                  /*int[] deletedPlayers = this.pmu.deletedPlayersThisRound();
                  if(deletedPlayers!=null){
                      this.tmu.prunePlayers(deletedPlayers);
                      this.pmu.resetDeletedPlayersArray();
                  }*/
                  return true;
              }
              else
                  return false;
          }              
          
      }
      else
          return false;
    }    
    
    //Announce Winner
    protected String announceWinner(){
        int[] winnersList = this.tmu.announceWinners();
        if(winnersList==null){
            return "No Winners - Winners list is empty";
        }
        else{
            return this.gmu.announceWinners(winnersList);
        }
    }
    protected int getWinner(){
        int[] winnersList = this.tmu.announceWinners();
        if(winnersList==null){
            return 0;
        }
        else{
            if(winnersList.length!=1) return -1;
            else return winnersList[0];
        }
    }
    protected int getRounds(){return this.dmu.getRoundCounter();}
    
    
}
