/*
 * DynamicCaseDialog.java
 *
 * Created on 11 September 2008, 13:36
 *
 * extends JDilaog to perform the single case selection dialog
 * @author maya
 */

package GUIComponents;
//java packages
import javax.swing.JPanel;
import javax.swing.*;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JButton;
import java.beans.*; //property change stuff
import java.awt.*;
import java.awt.event.*;

public class DynamicCaseDialog extends JDialog
                               implements ActionListener{
    private GameDictionary gameDictionary=null;
    private String[] currentCase=null;
    private JLabel[] labels=null;
    private JComboBox[] comboBoxes=null;    
    private JButton caseDoneJButton;
    private JButton caseCancelJButton;
    private JPanel panel;
    
    /** Creates a new instance of DynamicCaseDialog */
    public DynamicCaseDialog(Frame aFrame, GameDictionary dictionary) {
        super(aFrame,"Case Selector",false); 
        this.gameDictionary = dictionary;
        this.setAttributes(dictionary);
        setDefaultCloseOperation(HIDE_ON_CLOSE);

        //add components
        this.panel = new JPanel();
        this.panel.setLayout(new SpringLayout());
        this.caseDoneJButton = new JButton("OK");
        this.caseCancelJButton = new JButton("Cancel");
        this.caseDoneJButton.addActionListener(this);
        this.caseCancelJButton.addActionListener(this);
        for(int i=0; i<this.labels.length; i++){
            this.panel.add(this.labels[i]);
            this.panel.add(this.comboBoxes[i]);
        }
        this.panel.add(this.caseDoneJButton);    
        this.panel.add(this.caseCancelJButton);    
        SpringUtilities.makeCompactGrid(this.panel,
                                 this.labels.length+1, 2, //rows, cols
                                 5, 5, //initialX, initialY
                                 5, 5);//xPad, yPad


        this.setContentPane(panel);
    }
    
    
    public void actionPerformed(ActionEvent e) {
        if(e.getActionCommand()=="OK"){
            this.setCurrentCase();
            this.hide();            
        }
        else{
            this.gameDictionary=null;
            this.currentCase=null;
            this.hide(); 
        }
    }
    private void setAttributes(GameDictionary dictionary){
        String[] labelsValues = dictionary.getNonClassAttributes();
        this.labels = new JLabel[labelsValues.length];
        this.comboBoxes = new JComboBox[labelsValues.length];
        //set the labels values
        for(int i=0; i<this.labels.length; i++){
            labels[i] = new JLabel(labelsValues[i],JLabel.TRAILING);
        }
        //set the comboBoxes
        for(int i=0; i<this.comboBoxes.length; i++){
            String[] attributes = dictionary.getAttributes(i);
            this.comboBoxes[i] = new JComboBox(new javax.swing.DefaultComboBoxModel(attributes));
            this.labels[i].setLabelFor(this.comboBoxes[i]);
        }
    }
    private void setCurrentCase(){
        this.currentCase = new String[this.labels.length];
        for(int i=0; i<this.labels.length; i++){
            this.currentCase[i] = (String)this.comboBoxes[i].getSelectedItem();            
        }        
    }
    public String[] getCurrentCase(){return this.currentCase;}
}
