/*
 * FocusingUnit.java
 *
 * Created on 03 September 2008, 12:50
 *
 * Provides the data needed by the various focusing levels
 * @author maya
 */

package GUIComponents;
//java packages
import java.util.*;

public class FocusingUnit {
    ArgTree argTree;
    /** Creates a new instance of FocusingUnit, to focus new argumentatio tree*/
    public FocusingUnit(ArgTree treeTobeFoucsed) {
        this.argTree = treeTobeFoucsed;
    }
    
    //Methods
    /*------ FOCUSING METHODS --------*/
    /*1) strongest position algorithm
      2) focusing the strongest position*/          
    
    //STRONGEST POSITION METHODS
    /**returns the stronges green leafs (sharing the highest confidence)*/
    private ArgTreeNode[] strongestGreenLeafs(){
        if(this.argTree.activeGreenNodePlayer()==-1) return null;
        ArgTreeNode[] strongestGreenLeafs = null;
        double activeGreeConf = this.argTree.getGreenConf();
        ArgTreeNode[] leafs = this.argTree.getLeafs();
        for(int i=0; i<leafs.length; i++){
            if(leafs[i].getConfidence()>=activeGreeConf)
                strongestGreenLeafs=BasicOperations.append(strongestGreenLeafs, leafs[i]);
        }
        return strongestGreenLeafs;
    }
    /**returns the stronges blue leafs (blue dominant)*/
    private ArgTreeNode[] strongestBlueLeafs(){
        ArgTreeNode[] leafs = this.argTree.getLeafs();
        if(leafs.length==1) return leafs;
        ArgTreeNode[] strongestBlueLeafs=null;
        int[] blueNodes=this.argTree.countBlueNodes();
        if(blueNodes==null)return null;
        //set Max Blue
        int maxBlue=blueNodes[0];
        for(int i=1; i<blueNodes.length; i++){
            if(blueNodes[i]>maxBlue)
                maxBlue=blueNodes[i];
        }
        if (maxBlue==0) return null;
        //compose the strongest position        
        for(int i=0; i<leafs.length; i++){
            if(blueNodes[(leafs[i].getPlayer()-1)]==maxBlue)
              strongestBlueLeafs=BasicOperations.append(strongestBlueLeafs, leafs[i]);
        }
        return strongestBlueLeafs;
    }
    
    /**returns true if node's leafs are in the strongesPosition*/
    protected boolean hasAleafIn(ArgTreeNode node, ArgTreeNode[] strongestPosition){
        if(node==null)return false;
        if(node.getAttacks()==null) return false;
        ArgTreeNode[] nodeLeafs = argTree.leafs(node);
        if(ArgTreeNode.intersects(nodeLeafs,strongestPosition)!=null) return true;
        else return false;
    }
    /**returns true if one of the node's leafs equals in the node strongesPosition*/
    private boolean hasAleaf(ArgTreeNode node, ArgTreeNode strongestPosition){
        if(node==null)return false;
        if(node.getAttacks()==null) return false;
        ArgTreeNode[] nodeLeafs = argTree.leafs(node);
        if(strongestPosition.in(nodeLeafs)) return true;
        else return false;
    }
    
    //FOCUSED METHODS
    /**returns the strongest position on the tree*/
    protected ArgTreeNode[] strongestPosition(){
        if(this.argTree.hasGreenLeafs()) return this.strongestGreenLeafs();
        else    return this.strongestBlueLeafs();
    }
    
    //STRICTLY FOCUSED
    /**returns the focused Strongest Position on the tree using the Random method*/
    protected ArgTreeNode focusedStrongestPositionRandom(){
        ArgTreeNode [] strongestPosition = this.strongestPosition();
        if(strongestPosition.length==1) return strongestPosition[0];
        else{
            Random generator = new Random();
            int index = generator.nextInt(strongestPosition.length);
            return strongestPosition[index];
        }
    }    
    
    //CONTROLLED STRICTLY FOCUSED
    /**adds the round best move to the argumentation tree*/
    protected boolean addRoundBestMove(Move[] moves, int round){
        if (moves==null) return false;
        else{
            Move bestMove = this.selectBestMove(moves);
            return this.argTree.addRoundBestMove(bestMove,round);
        }
    }
    /**returns the best move in the list moves*/
    private Move selectBestMove(Move[] moves){
        Move bestMove = this.selectBestGreenMove(moves);
        if(bestMove!=null) return bestMove;
        else return this.selectBestBlueMove(moves);
    }
    /**returns the best green move in the list moves*/
    private Move selectBestGreenMove(Move[] moves){
        Move move=null;
        int firstGreenMoveIndx=0;
        while((firstGreenMoveIndx<moves.length)&&
                (MoveType.originalColour(moves[firstGreenMoveIndx].getType())!=Colours.GREEN))
            firstGreenMoveIndx++;
        if(firstGreenMoveIndx<moves.length){
            Move bestGreenMove = moves[firstGreenMoveIndx];
            for(int i=firstGreenMoveIndx+1; i<moves.length; i++){
                if(moves[i].getConfidence()>bestGreenMove.getConfidence())
                    bestGreenMove = moves[i];
            }
        }
        return move;
    }
    /**returns the best blue move in the list moves*/
    private Move selectBestBlueMove(Move[] moves){
        /*this is very simple interpertation of best blue moves
         a better solution would consult the tree*/
        int[] dominantBlues = this.argTree.dominantBlues();
        if(dominantBlues!=null){
            for(int i=0; i<moves.length; i++){
                for(int j=0; j<dominantBlues.length; j++){
                    if(moves[i].getSource()==dominantBlues[j])
                        return moves[i];
                }
            }        
        }
        for(int i=0; i<moves.length; i++){
            if(moves[i].getType()==3){
                return moves[i];
            }                    
        }
        return moves[0];

    }
    /*-------------------------------------------------------------------------------------------*/
}
