/*
 * GMU.java
 *
 * Created on 03 September 2008, 15:40
 *
 * Groups Management Unit GMU. Responsible for:
 1) managing the groups in the game
 2) managing the groups played moves history log
 3) recieving moves from groups every round and redirect those moves to the chairperson
 *
 *@author maya
 */

package GUIComponents;

//java packages
import java.util.Random;
import java.util.*;

public class GMU {   
    private ActiveGroupsList activeGroupsList; //the active groups in the unit.
    private HistoryLog historyLog; //the history log of the moves played in the game so far.
    private short[] currentCase;//the case under discussion.
    private short[] classifications;//the possible classifications of this case.
    
    /**default constructor*/
    public GMU() {
        this.activeGroupsList = null;
        this.historyLog = null;
    }
    /**main costructor: construct both active groups list and the history log*/
    public GMU(int maxNoGroups) {
        this.activeGroupsList = new ActiveGroupsList(maxNoGroups);
        this.historyLog = new HistoryLog(maxNoGroups);
    }
    
    //setters and getters
    protected void setClassifications(short[] newClassifications){this.classifications =newClassifications;}
    protected void setCurrentCase(short[] newCurrentCase){this.currentCase = newCurrentCase;}       
    protected ActiveGroupsListNode getActiveGroup(int groupIndx){return this.activeGroupsList.getActiveGroup(groupIndx);}   
    protected ActiveGroupsListNode getFirstGroup(){return this.activeGroupsList.getFirstGroup();}      
    protected void setOfflineHistoryLog(String path){historyLog.setOfflineHistoryLog(path);}
    protected GMUInterface createInterface(){ return new GMUInterface(this.activeGroupsList, this.historyLog);}
    protected short getGroupGoal(int groupIndx){
        return this.activeGroupsList.getGroupGoal(groupIndx);
    }
    /*-----------------------------------------------------------------------------*/
    //methods
    //ADD NEW GROUP - DELETE GROUP
    /**returns the set of counter goals of the input goal*/
    private short[] setCounterGoals(short goal){
       short[] counterGoals = new short[this.classifications.length-1];
       boolean found = false;
       int j=0;
       for(int i=0; i<classifications.length-1; i++){
           if(this.classifications[i] == goal)
               found = true;
           else{
               counterGoals[j] = this.classifications[i];
               j++;
           }
       }
       if(this.classifications[this.classifications.length-1] == goal)  found = true;       
       else{
           if(found) 
            {counterGoals[counterGoals.length-1] = this.classifications[this.classifications.length-1];}
       }
       
       if(found){return counterGoals;}
       else {return null;}        
    }
    /**adds a new group to the GMU, aims to proove the input goal*/
    protected boolean addNewGroup(short goal){        
        short [] counterGoals = this.setCounterGoals(goal);
        if(counterGoals != null){
            Group newGroup = new Group(goal,counterGoals);
            boolean result =  this.activeGroupsList.addNewGroup(newGroup);
            return result;
        }
        else
            return false;
    }
    /**adds a new group to the GMU, aims to proove the input goal, and has as players the input players*/
    protected boolean addNewGroup(short goal, GroupPlayer[] players){        
        short [] counterGoals = this.setCounterGoals(goal);
        if(counterGoals != null){
            Group newGroup = new Group(goal,counterGoals);
            if(players!=null){
                for(int i=0; i<players.length; i++){
                    newGroup.addPlayer(players[i]);
                }                    
            }
            boolean result =  this.activeGroupsList.addNewGroup(newGroup);
            return result;
        }
        else
            return false;
    }
    /**delete the given group*/
    protected boolean deleteGroup(int groupIndx){return this.activeGroupsList.deleteGroup(groupIndx);}
    /*-----------------------------------------------------------------------------*/
    //ORDER PLAYERS
    /**randomly ordering the groups in the GMU*/
    private void orderGroups(){
        this.activeGroupsList.order();
    }
    
    //INITIALIZE GAME
    /**check if the given goal belongs to the game settings*/
    private boolean goalOK(short goal){
        boolean found = false;
        int i=0;
        while((!found)&&(i<classifications.length)){
            if(this.classifications[i]==goal){found = true;}
            else{i++;}
        }
        return found;
    }

    
    /**initializes GMU*/
    protected int initialize(){
        this.activeGroupsList.setGroupsIndex();
        this.setGroupPlayers();
        this.activeGroupsList.initialize(this.currentCase);
        return this.activeGroupsList.getCurrentNoGroups();
    }
   
    //START GAME
    /**starts the game on the GMU level*/
    protected void startGame(){
        //this.orderGroups();
        this.activeGroupsList.setGroupsIndex();
        this.setGroupPlayers();
        //this.outputGroups();
    }
    /**sets the players in each group*/
    private void setGroupPlayers(){
        if(this.activeGroupsList!=null)
        this.activeGroupsList.setGroupPlayers();
    }
    /**plays first round*/
    protected Move firstRound(){
        Move firstMove = this.activeGroupsList.firstRound();
        int trials = 10;
        int i=1;
        while((firstMove==null)&&(i<trials)){
            this.orderGroups();
            this.setGroupPlayers();
            firstMove = this.activeGroupsList.firstRound();            
            i++;
        }
        if(firstMove!=null){
            this.historyLog.insertFirstMove(firstMove);            
        }
        
        return firstMove;
        //if the selected player failed in playing a move then update the flag and reshuffle the players
    }
    /*-----------------------------------------------------------------------------*/
    
    //GAME ROUNDS
    /**translate tree nodes in compressed format to moves*/
    private Move[] treeNodeToMoves(CompressedArgTreeNode[] leafs){
        Move [] nodeMoves = new Move[leafs.length];
        for(int i=0; i<leafs.length; i++){
           nodeMoves[i] = this.historyLog.findMove(leafs[i].getPlayer(), leafs[i].getParent(), leafs[i].getRound(), i); 
        }            
        return nodeMoves;
    }    
    /**resolves collidings: this function is used to resolve any colliding in the moves played in a given round
         i.e. if two players played similar rules at the same round.
         This problem can happen only if:
         1) two or more player played identical distinguishing moves.
         2) two or more players discovered some unwanted consequences in some player's move
         Therefore we use the following rule to resolve colliding incidences
         R1) unwanted consequences: pick random one
         R2) distinguish: pick the lower confidence distinguish*/
    private Move[] resCollidingsLevel1(Move[] roundMoves){        
        if(roundMoves.length ==1)
            return roundMoves;        
        else{            
            for(int i=0; i<roundMoves.length; i++){
                if(roundMoves[i]!=null){
                    Move candidate = roundMoves[i];
                    for(int j=i+1; j<roundMoves.length; j++){
                        if(roundMoves[j]!=null){
                            if(Move.collides(roundMoves[i], roundMoves[j])){                                
                                if(roundMoves[i].getType()==3){//unwanted conseq
                                    System.out.println("resolve colliding unwanted consequences moves");
                                    roundMoves[j]=null;
                                }
                                else{//distinguish
                                    System.out.println("resolve colliding distinguish moves");
                                    if(roundMoves[j].getConfidence()<roundMoves[i].getConfidence()){
                                        roundMoves[i]=null;
                                        break;
                                    }
                                    else{
                                        roundMoves[j]=null;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            Move[] uncollidedMoves = this.compressMoves(roundMoves);
            return uncollidedMoves;
        }
    }
    /**returns the input moves in compressed format*/
    private Move[] compressMoves(Move[] roundMoves){        
        Move[] compressedMoves = null;
        for(int i=0; i<roundMoves.length; i++){
            if(roundMoves[i]!=null)
                compressedMoves = BasicOperations.append(compressedMoves,roundMoves[i]);
        }
        return compressedMoves;
    }
    /**resolves collidings: this function is used to resolve any colliding in the moves played in a given round
         i.e. if two players played similar rules at the same round.
         This problem can happen only if:
         1) two or more player played similar distinguishing moves. (i.e. used the same case 
         attributes to lower the confidence.
         2) two or more players discovered some unwanted consequences in some player's move
         Therefore we use the following rule to resolve colliding incidences
         R1) unwanted consequences: choose the move that threaten the original attacked move the most!
         R2) distinguish: pick the lower confidence distinguish*/
    private Move[] collidingsRESLevel2(Move[] roundMoves){        
        Move[] uncollidedMoves = null;
        return uncollidedMoves;
    }
    
    /*-----------------------------------------------------------------------------*/
    //STRATEGY GAMES
    /**plays next round (not 1st)*/
    protected Move[] nextRound(int round, TMUInterface tmuInterface, short gameStatus){
        //gameStatus 0=ok 1=danger
        GMUInterface gmuInterface = this.createInterface();
        Move[] roundMoves= this.activeGroupsList.nextRound(round,tmuInterface, gmuInterface ,gameStatus);
        if(roundMoves!=null)
            roundMoves = this.resCollidingsLevel1(roundMoves);
        Move[] successfulMoves=null;        
        if(roundMoves!=null){
            for(int i=0; i<roundMoves.length; i++){
                int insertToHistory = this.historyLog.insertMove(roundMoves[i],round);
                if(insertToHistory==0){
                   successfulMoves = BasicOperations.append(successfulMoves, roundMoves[i]);
                }
                else{
                    System.out.print("Cant add this move to the game history because : ");
                    switch(insertToHistory){
                        case 1:
                            System.out.println("source player used similar move against same target");
                            break;
                        case 2:
                            System.out.println("source player used similar move against different target");
                            break;
                        case 3:
                            System.out.println("another player used similar move against same target");
                            break;
                        case 4:
                            System.out.println("it is similar to the first move");
                            break;                            
                    }
                }
            }
        }       
        return successfulMoves;
    }
    /*-----------------------------------------------------------------------------*/
    
    /**Delets groups after failing in playing moves in n continous rounds*/
    protected int[] deletedGroupsThisRound(){
        boolean deletedGroups = this.activeGroupsList.deleteGroups();
        if (deletedGroups){
            return this.deletedGroupsThisRound();
        }
        else
            return null;
    }
    /**resets the deleted groups array (after completing the deleting process*/
    protected void resetDeletedGroupsArray(){this.activeGroupsList.resetDeletedGroupsArray();}
    //public void outputGroups(){this.activeGroupsList.output()}
    
    /**announce the winner of the game*/
    protected String announceWinners(int[] winnersList){
        String output;
        if(winnersList.length==1){
            output = "THE WINNER IS\n";
            output+=this.activeGroupsList.output(winnersList[0]);
        }
        else{
            output = "THERE is more than ONE WINNER\n";
            output += "WINNERS ARE\n";
            for(int i=0; i<winnersList.length; i++){
                int winner = winnersList[i];
                output += this.activeGroupsList.output(winnersList[i]);
            }
        }
        return output;
    }
   protected InGroupMove[][] inGroupMoves(int groupIndx){return this.activeGroupsList.inGroupMoves(groupIndx);} 
}