/*
 * GameSettings.java
 *
 * Created on 03 September 2008, 15:29
 *
 * the basic Game settings class, stores the game information as chosen by the user
 *
 * @author maya
 */

package GUIComponents;

public class GameSettings {
    private int maxTreeSize;//the maximum number of nodes per tree
    private int maxNoGroups;//the maximum number of groups in a game
    private int maxNoRounds;//the maximum number of rounds in a game
    private int currentNoPlayers;//the number of players in the current game
    private int maxNoSkips;//the maximum number of rounds each group allowed to skip
    private double minConf=70;//the maximum confidence for accepting a rule as valid rule
    private double maxConf=100;//the maximum confidence allowed in the game
    private int[] defaultValues = {100,100,3,2};//tree, rounds, groups, skips
    
    /** Creates a new instance of GameSettings */
    public GameSettings() {
        this.maxTreeSize = 100;
        this.maxNoRounds = 100;
        this.maxNoGroups = 3;
        this.currentNoPlayers = 0;
        this.maxNoSkips = 2;  
    }
    
    /**setters & getters*/
    protected int getMaxTreeSize(){return this.maxTreeSize;}
    protected int getMaxNoGroups(){return this.maxNoGroups;}
    protected int getMaxNoRounds(){return this.maxNoRounds;}
    protected int getCurrentNoPlayers(){return this.currentNoPlayers;}
    protected int getMaxNoSkips(){return this.maxNoSkips;}
    protected double getMinConf(){return this.minConf;}
    protected double getMaxConf(){return this.maxConf;}
    protected void setMaxTreeSize(int newTreeSize){this.maxTreeSize = newTreeSize;}
    protected void setMaxNoRounds(int newMaxNoRounds){this.maxNoRounds = newMaxNoRounds;}
    protected void setMaxNoGroups(int newNoGroups){this.maxNoGroups = newNoGroups;}
    protected void setCurrentNoPlayers(int newCurrentNoPlayers){this.currentNoPlayers = newCurrentNoPlayers;}
    protected void setMaxNoSkips(int newMaxNoSkips){this.maxNoSkips = newMaxNoSkips;}
    protected void setMinConf(double newMinConf){this.minConf = newMinConf;}
    protected void setMaxConf(double newMaxConf){this.minConf = newMaxConf;}    
    protected int[] getDefaultParameters(){return this.defaultValues;}
    protected int[] getGameParameters(){
        int [] gameParameters = new int[4];
        gameParameters[0] = this.getMaxTreeSize();
        gameParameters[1] = this.getMaxNoRounds();
        gameParameters[2] = this.getMaxNoGroups();
        gameParameters[3] = this.getMaxNoSkips();
        return gameParameters;
    }
    //methods
    /**increases the number of the current players*/
    public void increaseCurrentPlayer(int increase){
        this.currentNoPlayers+=increase;
    }
    /**decreases the number of the current players*/
    public void decreaseCurrentPlayer(int decrease){
        if (decrease>this.currentNoPlayers)
            this.currentNoPlayers=0;
        else
            this.currentNoPlayers-=decrease;
    }
}
