/*
 * Group.java
 *
 * Created on 03 September 2008, 15:37
 *
 * Groups class: comprises of a number of players sharing the same objective
 * each group has a leader players[0], and a type: homogenous or heterogenous
 *
 *@author maya
 */

package GUIComponents;

public class Group {
    int numberOfPlayers=0;
    GroupPlayer [] players=null; //the leader is always players[0];
    int index;//group index in the GMU.    
    short goal;//the common goal of the players in the group
    private short[] counterGoals;//the counter goals (classifications - goal)    
    boolean type; //The group type: false = hetero, true=homo
    /*contains the detailed moves played in the group, last entry is the leader choice*/
    InGroupMove[][] inGroupMoves = null;
    
    //constructor
    public Group(short goal, short[] counterGoals) {
        this.goal = goal;
        this.counterGoals = counterGoals;
    }
    
    public Group(short goal, short[] counterGoals, GroupPlayer[] players){
        this.goal =goal;
        this.counterGoals = counterGoals;
        if(players!=null){
                for(int i=0; i<players.length; i++){
                    this.addPlayer(players[i]);
                }                    
            }
    }
   
    //setters & getters
    protected int getNumberOfPlayers(){return this.numberOfPlayers;}
    protected short getGoal(){return this.goal;}
    protected void setIndex(int newIndex){this.index = newIndex;}
    protected int getIndex(){return this.index;}
    protected boolean getType(){return this.type;}
    protected GroupPlayer[] getPlayers(){return this.players;}
    protected InGroupMove[][] getInGroupMoves(){return this.inGroupMoves;}
    //methods    
    
    //BASIC METHODS    
    public void initialize(short[] currentCase){
        if(this.players!=null){            
            for(int i=0; i<this.players.length; i++)
                this.players[i].initialize(currentCase);
            this.orderPlayers();
            this.setPlayersIndex();
            this.checkType();
            this.players[0].setLeader();            
            this.players[0].setLeadershipType(this.type);            
            this.outputLeader();
        }
    }
    private void outputLeader(){
        System.out.print("Group - " + this.goal + " leader is : ");
        this.players[0].output();
    }
    /*this procedure ranks the strategy of each player, 
     and order players according to their strategy type*/
    private void orderPlayers(){
        if((this.players!=null)&&((this.players.length!=1))){      
            for(int i=1; i<this.players.length; i++){                
                GroupPlayer temp = this.players[i];
                int rank = temp.strategyRank();
                int j=i;
                while ((j > 0) && (this.players[j-1].strategyRank()>rank)){
                    this.players[j] = this.players[j-1];
                    j--;
                }
                if(j>0){
                    int k=j;                    
                    while ((k > 0) && (this.players[k-1].getExperience()<this.players[j].getExperience())){
                    this.players[k] = this.players[k-1];
                    k--;
                    }
                    this.players[k] = temp;
                }
                else{
                   this.players[j]=temp; 
                }
            }
        }                
    }
    /*set players index to be the same as the group index*/
    public void setPlayersIndex(){
        if(this.players!=null){
            for(int i=0; i<this.players.length; i++)
                this.players[i].setPlayerIndx(this.index);
        }        
    }
    /*check if the group is homogenous or heterogenous and update the group type*/
    private void checkType(){
        if((this.players!=null)&&((this.players.length!=1))){
            int rank = this.players[0].strategyRank();
            for(int i=1; i<this.players.length; i++){
                if(this.players[i].strategyRank()!=rank){
                    this.type=false;
                    break;
                }
            }
            this.type=true;
        }
    }
    
    //ADD DELETE and FIND PLAYERS in the group
    public void addPlayer(GroupPlayer newPlayer){
        if (this.players==null){
            this.players=new GroupPlayer[1];
            this.players[0] = newPlayer;
            this.numberOfPlayers++;
        }
        else{
           GroupPlayer [] newPlayers = new GroupPlayer[this.players.length+1];
           for(int i=0; i< players.length; i++)
               newPlayers[i] = this.players[i];
           newPlayers[players.length] = newPlayer;
           this.players = newPlayers;
           this.numberOfPlayers++;
        }            
    }
    private void deletePlayer2(int playerIndx){
        GroupPlayer[] newPlayers = new GroupPlayer[this.players.length-1];
        int j=0;
        for(int i=0; i<players.length; i++){
            if(this.players[i].getplayerIndx()!=playerIndx){
               newPlayers[j] = this.players[i];
               j++;
            }
        }
        this.players=newPlayers;
    }
    public void deletePlayer(int playerIndx){
       int groupIndx = this.findPlayer(playerIndx);
       if(groupIndx!=-1)
           this.deletePlayer2(playerIndx);
    }
    public int findPlayer(int playerIndx){       
        for(int i=0; i<players.length; i++){
            if(this.players[i].getplayerIndx()==playerIndx)
                return i;
        }
        return -1;
    }
    
    //Play in Normal Mode
    public Move playFirstMove(){
        if(this.numberOfPlayers==1){
            Move firstMove= this.players[0].playFirstMove();
            //this.outputFirstMove(firstMove);
            InGroupMove ingroupMove = new InGroupMove(firstMove, 1);
            this.inGroupMoves = BasicOperations.append(this.inGroupMoves, ingroupMove);
            return firstMove;
        }
        else{
            Move[] firstMoves = new Move[this.numberOfPlayers];
            for(int i=0; i<this.numberOfPlayers; i++){
                firstMoves[i] = this.players[i].playFirstMove();
                //this.outputPlayerFirstMove(firstMoves[i], i);
            }
            Move firstMove= this.makeDecisionFirstMove(firstMoves);            
            this.addIngroupMove(firstMoves, firstMove);
           // this.outputFirstMove(firstMove);
            return firstMove;
        }
    }    
    public Move playNextRound(int round, TMUInterface tmuInterface, 
                                 GMUInterface gmuInterface, short gameFlag, short statusFlag){
        if(this.numberOfPlayers==1){
            Move move= this.players[0].playNextRound(round,tmuInterface,gmuInterface, gameFlag,statusFlag);
            //this.outputMove(move);
            InGroupMove ingroupMove = new InGroupMove(move, 1);
            this.inGroupMoves = BasicOperations.append(this.inGroupMoves, ingroupMove);            
            return move;
        }
        else{
            Move[] roundMoves = new Move[this.numberOfPlayers];
            for(int i=0; i<numberOfPlayers; i++){
                roundMoves[i] = this.players[i].playNextRound(round,tmuInterface,gmuInterface, gameFlag,statusFlag);
                //this.outputPlayerMove(roundMoves[i], i);
            }
            Move move= this.makeDecision( round,  tmuInterface, gmuInterface, gameFlag,  statusFlag, roundMoves);
            this.addIngroupMove(roundMoves, move);
            //this.outputMove(move);
            return move;
        }
    }
   
    //DECISION MAKING UNIT
    private Move makeDecision(int round, TMUInterface tmuInterface, GMUInterface gmuInterface,
                               short gameFlag, short statusFlag, Move[] roundMoves){
        return this.players[0].decide(roundMoves,round,tmuInterface,gmuInterface,statusFlag,gameFlag);
    }    
    private Move makeDecisionFirstMove(Move[] firstMoves){
        return this.players[0].decideFirstMove(firstMoves);
    }
    
    //OUTPUT
    private String outputFirstMove(Move move){       
        String output;
        if(move!=null){             
             output = "Group  - " + this.goal+"\n";
             output+="First Move\n";
             output+=move.getRule().output();
        }
        else
            output = "Group  - " + this.goal +"Has failed in proposing first move\n";
        return output;
    }
    private String outputMove(Move move){       
        if(move!=null){
            String output = "Group  - " + this.goal+"\n";
             switch(move.getType()){
                 case MoveType.PROPOSE_RULE:
                    output+="Propose rule\n"; 
                    break;
                 case MoveType.DISTINGUISH:
                    output+="Distinguish\n"; 
                    break;             
                 case MoveType.UNWANTED_CONSEQUENCES:
                    output+="Unwanted consequences\n"; 
                    break;              
                 case MoveType.COUNTER_RULE:
                    output+="Counter rule\n"; 
                    break;                    
                 case MoveType.INCREASE_CONFIDENCE:
                    output+="Increase Confidence\n";  
                    break;                    
                 case MoveType.WITHDRAW_UNWANTED_CONSEQUENCES:
                    output+="Withdraw unwanted consequences\n";  
                    break;                    
             }
             output+=move.getRule().output();
             return output;
        }
        else
            return "Group  - " + this.goal +"Has failed in playing a move\n";
    }
    private String outputPlayerFirstMove(Move move, int playerIndx){       
        String output;
        if(move!=null){             
             output = "Player  - " + playerIndx+"\n";
             output+="First Move\n";
             output+=move.getRule().output();
        }
        else
            output = "Player  - " + playerIndx +"Has failed in proposing first move\n";
        return output;
    }
    private String outputPlayerMove(Move move,int playerIndx){       
        if(move!=null){
            String output = "Player  - " + playerIndx+"\n";
             switch(move.getType()){
                 case MoveType.PROPOSE_RULE:
                    output+="Propose rule\n"; 
                    break;
                 case MoveType.DISTINGUISH:
                    output+="Distinguish\n"; 
                    break;             
                 case MoveType.UNWANTED_CONSEQUENCES:
                    output+="Unwanted consequences\n"; 
                    break;              
                 case MoveType.COUNTER_RULE:
                    output+="Counter rule\n"; 
                    break;                    
                 case MoveType.INCREASE_CONFIDENCE:
                    output+="Increase Confidence\n";  
                    break;                    
                 case MoveType.WITHDRAW_UNWANTED_CONSEQUENCES:
                    output+="Withdraw unwanted consequences\n";  
                    break;                    
             }
             output+=move.getRule().output();
             return output;
        }
        else
            return "Group  - " + this.goal +"Has failed in playing a move\n";
    }
    public String output(){
        String output = "Group Index: " + this.index + "\n";
        output+="Goal: " + this.goal+ "\n";
        return output;
    }
    
    //ingroupMoves
    private void addIngroupMove(Move[] moves, Move leaderChoice){      
        Move[] newMoves = BasicOperations.append(moves, leaderChoice);
        if((newMoves!=null)&&(leaderChoice!=null)){        
            InGroupMove[] ingroupMoves = new InGroupMove[newMoves.length];        
            for(int i=0; i<moves.length; i++){            
                ingroupMoves[i] = new InGroupMove(moves[i],i+1);
            }
            int leaderchoiceIndx = this.getLeaderChoicePlayerIndx(moves, leaderChoice)+1;
            ingroupMoves[moves.length] = new InGroupMove(leaderChoice,leaderchoiceIndx);
            this.inGroupMoves = BasicOperations.append(this.inGroupMoves, ingroupMoves);
        }
        else{
            InGroupMove[] ingroupMoves=null;
            this.inGroupMoves = BasicOperations.append(this.inGroupMoves, ingroupMoves);
        }
    }

    private int getLeaderChoicePlayerIndx(Move[] moves, Move leaderChoice){
        if(moves==null) return -1;
        if (leaderChoice== null) return -1;
        for(int i=0; i<moves.length; i++){
            if(Move.equals(moves[i], leaderChoice)) return i;
        }
        return -1;
    }
    
}
