/*
 * GroupPlayer.java
 *
 * Created on 03 September 2008, 15:33
 *
 * GroupPlayer class represents members of groups, single players are considered as one player groups
 * GroupPlayer has the following functions:
 * 1) mining AR from background database
 * 2) selects the best availbe move in each round according to a strategy
 * 3) if a group leader, then the player is responsible for selecting the group moves
 * 
 *@author maya
 */

package GUIComponents;

//java packages
import java.util.Arrays;

public class GroupPlayer {
   private int playerIndx; //the Index of this player's group in the GMU   
   private Strategy strategy;//the player's strategy in this game   
   private RulesMiningUnit rmu=null;
   private LeadershipUnit leadershipUnit = null;/*if this unit is not null, then this player is the leader of its group*/
   
   /**basic constructor*/
    public GroupPlayer(String dataFile, short goal, short[] counterGoals, double minSupport) {
        rmu = new RulesMiningUnit(dataFile,goal,counterGoals,minSupport);
        this.playerIndx=-1;        
        this.strategy = new Strategy();
    }
    
    //setters & getters
    protected void setCurrentCase(short[] inputCase){this.rmu.setCurrentCase(inputCase);}    
    protected void setPlayerIndx(int newIndx){this.playerIndx = newIndx;}    
    protected short getGoal(){return this.rmu.getGoal();}
    protected short[] getCounterGoals(){return this.rmu.getCounterGoals();}
    protected String getDataFilePath(){return this.rmu.getDataFilePath();}
    protected int getplayerIndx(){return this.playerIndx;}
    protected Strategy getStrategy(){return this.strategy;}
        
    /*SET STRATEGY FUNCTIONS*/
    protected void setSmartnessLevel(int smartnessLevel){this.strategy.setSmartness(smartnessLevel);}    
    protected void setStrategyMode(int mode){this.strategy.setStrategyMode(mode);}    
    protected void setStrategyType(int type){this.strategy.setStrategyType(type);}
   
   //methods
   //BASICS
   /*create the P-Tree that represents the data stored in the dataFile*/
   private void GetData(){
       int numberOfRows = this.rmu.GetData();
        this.strategy.setExperience(numberOfRows);
    }   
   /*create the T-Tree of the instance P-Tree*/
   private int buildTTree(){
	int maxLevel =  this.rmu.buildTTree();
	this.strategy.setMaxLevel(maxLevel);
	return maxLevel;
    }   
   
   //INITIALIZE THE PLAYER FOR THE GAME
   /**initializes the rule mining unit*/
   protected void initialize(short[] currentCase){
       this.GetData();
       this.buildTTree();
       this.rmu.initialize(currentCase);
   }
   /**insert a new rule into the set of playedRules*/
   private void insertRuleintoRulelist(RuleNode rule){this.rmu.insertRuleintoRulelist(rule);}
   
   //LEADERSHIP METHODS
   /**sets the player as leader of its group*/
   protected void setLeader(){
       this.leadershipUnit=new LeadershipUnit(this.strategy.strategyRank(),this.playerIndx);
   }
   /**sets the type of the player's leadrship*/
   protected void setLeadershipType(boolean type){
       if(this.leadershipUnit!=null)
           this.leadershipUnit.setLeadershipType(type);
   }
   /**returns true if the player is a leader of its group*/
   protected boolean isLeader(){
       if(this.leadershipUnit!=null)return true;
       else return false;
   }
   /*---------------------------------------------------------------------------------------*/
   
   //PLAY METHODS
   public Move playFirstMove(){        
       Move move = this.rmu.proposeFirstMove(this.strategy);
       if(move!=null){
           this.rmu.insertRuleintoRulelist(move.getRule());
       }
       return move;
    }
   public Move playNextRound(int round, TMUInterface tmuInterface, 
                                 GMUInterface gmuInterface, short gameFlag, short statusFlag){
       int x=0;
       if(round ==5)
           x = 7;
       Move move = null;       
       switch(this.strategy.getStrategyType()){
           case 1://attack whenever possible
               move = this.attackWhenEverPossible(round, tmuInterface, gmuInterface, gameFlag, statusFlag);
               break;
           case 2://attack whenever needed
               move = this.attackWhenNeeded(round, tmuInterface, gmuInterface, gameFlag, statusFlag);
               break;
           case 3://attack to prevent threat
               move = this.attackToPreventThreat(round, tmuInterface, gmuInterface, gameFlag, statusFlag);
               break;
       }       
       return move;       
   }
   
   //PLAY DUMP AND DUMPER (NON STRATEGIC FUNCTIONS)
   private Move playDump(Move[] leafMoves){
        Move move =null;
        int i=0;
        while ((move==null)&&(i<leafMoves.length)){
            if(leafMoves[i] == null){
                System.out.println("error Move index - " + i);
                System.out.println("error Move Leafs - " + leafMoves.length);
                i++;
            }
            else
            {       
                if(this.strategy.getStrategyMode()==1)
                    move = this.attackDestroy(leafMoves[i]);
                else
                    move = this.attackBuild(leafMoves[i]);
                if(move==null)
                        i++;
                /*else
                    i++;*/
            }
        }
        return move;
    }
   private Move[] threatMoves(Move[] leafMoves){
       Move[] threatMoves = null;
       if(leafMoves!=null){
        for(int i=0; i<leafMoves.length; i++){
            if(leafMoves[i]!=null){
                if(leafMoves[i].getTarget()==this.playerIndx)
                    threatMoves = BasicOperations.append(threatMoves, leafMoves[i]);
            }
            else
            {
                int x;
            }
        }
       }
       return threatMoves;
   }
   private Move[] otherMoves(Move[] leafMoves){
       Move[] threatMoves = null;
       if(leafMoves!=null){
        for(int i=0; i< leafMoves.length; i++){
            if(leafMoves[i].getTarget()!=this.playerIndx)
                threatMoves = BasicOperations.append(threatMoves, leafMoves[i]);
        }
       }
       return threatMoves;
   }
   private Move playLevel2(Move[] leafMoves){
        Move move =null;
        Move[] ThreatMoves = this.threatMoves(leafMoves);        
        move = this.playLevel1(ThreatMoves);
        if(move==null)
        {
            Move[] otherMoves = this.otherMoves(leafMoves);
            if(otherMoves!=null)
                move = this.playLevel1(otherMoves);
            
        }
        return move;
    }   
   private Move playLevel1(Move[] leafMoves){
        Move move =null;
        if(leafMoves!=null){
            Move[] orderedLeafMoves = this.orderLeafMoves(leafMoves);
            int i=0;
            while ((move==null)&&(i<orderedLeafMoves.length)){
                if(orderedLeafMoves[i] != null){
                    if(this.strategy.getStrategyMode()==1)
                        move = this.attackDestroy(leafMoves[i]);
                    else
                        move = this.attackBuild(leafMoves[i]);
                    if(move==null)
                            i++;
                    else
                        i++;
                }
            }
        }
        return move;
    }
   private Move[] orderLeafMoves(Move[] leafMoves){       
       Move [] orderedLeafMoves = new Move[leafMoves.length];
       orderedLeafMoves[0] = leafMoves[0];
       for(int i=1; i< leafMoves.length; i++){
           int position = i;
           int shifts = 0;
           boolean positionFound = false;
           while((position>0)&&(!positionFound)){
               if(orderedLeafMoves[position-1].getConfidence()>=leafMoves[i].getConfidence()){
                   positionFound = true;
                   if(shifts==0){
                       orderedLeafMoves[i] = leafMoves[i];
                   }
                   else{ 
                       int k=i;
                       for(int j=0; j<shifts; j++){
                         orderedLeafMoves[k]   = orderedLeafMoves[k-1];
                         k--;
                       }
                       orderedLeafMoves[k] = leafMoves[i];
                   }
               }
               else
               {
                   shifts++;
                   position --;
               }
               if((position==0)&&(!positionFound)){
                   int k=i;
                   for(int j=0; j<shifts; j++){
                       orderedLeafMoves[k]   = orderedLeafMoves[k-1];
                       k--;
                   }
                   orderedLeafMoves[k] = leafMoves[i];
               }
           }       
       }
       return orderedLeafMoves;
   }
   
   /*The Official Three Levels*/
   /*1*/
   private Move attackWhenEverPossible(int round, TMUInterface tmuInterface, GMUInterface gmuInterface, short gameFlag,short statusFlag){
       if(this.strategy.getSmartness()==2){
           return this.attackWhenEverPossibleTree(round, tmuInterface, gmuInterface,gameFlag,statusFlag);
       }
       else{
           switch (this.strategy.getStrategyMode()){
               case 1://build
                   return this.attackWhenEverPossibleBuild(round,tmuInterface,gmuInterface,gameFlag, statusFlag);
               default://destroy
                   return this.attackWhenEverPossibleDestroy(round,tmuInterface,gmuInterface,gameFlag, statusFlag);                   
           }
       }
   }
   private Move attackWhenEverPossibleBuild(int round, TMUInterface tmuInterface, GMUInterface gmuInterface, short gameFlag,short statusFlag){
       CompressedArgTreeNode[] compressedLeafs = tmuInterface.requestCompressedLeafs();
       Move[] leafMoves = gmuInterface.leafsToMoves(compressedLeafs);
       if (this.strategy.getSmartness()==0)
           return attackWhenEverPossibleBuildBlind(round, leafMoves, gameFlag,statusFlag);
       else
           return attackWhenEverPossibleBuildFocused(round, leafMoves, gameFlag,statusFlag);  
   }
   private Move attackWhenEverPossibleDestroy(int round, TMUInterface tmuInterface, GMUInterface gmuInterface, short gameFlag,short statusFlag){
       CompressedArgTreeNode[] compressedLeafs = tmuInterface.requestCompressedLeafs();
       Move[] leafMoves = gmuInterface.leafsToMoves(compressedLeafs);       
       if (this.strategy.getSmartness()==0)
           return attackWhenEverPossibleDestroyBlind(round, leafMoves, gameFlag,statusFlag);
       else
           return attackWhenEverPossibleDestroyFocused(round, leafMoves, gameFlag,statusFlag);  
   }
   private Move attackWhenEverPossibleTree(int round, TMUInterface tmuInterface, GMUInterface gmuInterface, short gameFlag,short statusFlag){
    /*attack when ever possible Tree Oriented
    *1) Try to propose a rule with higher confidence than the green conf -  
    *   direct this move against the highest conf leaf node (if not the green conf).
    *2) Try to propose a rule with higher confidence than the active green conf - 
    *   direct this move against the highest conf leaf node (if not the active green conf).
    *3) Try to attack the green conf- (path,leaf) (using the original function)
    *4) Try to attack the active green conf (using the original function)
    *5) If adding a blue node to the game make the player dominant blue then try to play blue node
    *6) Try to attack when ever possible Focused Mode*/
       Move move=null;
       /*1*/
       if(tmuInterface.greenConfStatus()){//green conf
               if(tmuInterface.greenConfPlayer()!= this.playerIndx){
                   double greenConf = tmuInterface.greenConf();
                   move = this.attackConfidence(round, greenConf, tmuInterface, gmuInterface);
                   if(move!=null)
                       return move;
           }
       }
       /*2*/
       if(!tmuInterface.activeGreenEqualsGreenConf()){
           if(tmuInterface.activeGreenConfStatus()){//green conf 
               if(tmuInterface.activeGreenNodePlayer()!= this.playerIndx){
                   double activeGreenConf = tmuInterface.activeGreenConf();
                   move = this.attackConfidence(round, activeGreenConf, tmuInterface, gmuInterface);
                   if(move!=null)
                       return move;
               }
           }
        }
       /*3*/
       if(this.playBlue(round, tmuInterface,gmuInterface)){
         CompressedArgTreeNode[] compressedLeafs = tmuInterface.requestCompressedLeafs();
         Move[] leafMoves = gmuInterface.leafsToMoves(compressedLeafs);
         for(int i=0; i<leafMoves.length; i++){         
             move =this.attackDestroyOnly(leafMoves[i]);
             if(move!=null){
                 return move;
             }
         }
       }           
       /*4*/
       CompressedArgTreeNode[] compressedLeafs = tmuInterface.requestCompressedLeafs();
       Move[] leafMoves = gmuInterface.leafsToMoves(compressedLeafs);
       if(this.strategy.getStrategyMode()==1)
           return this.attackWhenEverPossibleBuildFocused(round,leafMoves,gameFlag,statusFlag);
       else
           return this.attackWhenEverPossibleDestroyFocused(round,leafMoves,gameFlag,statusFlag);      
   }
   private boolean playBlue(int round, TMUInterface tmuInterface, GMUInterface gmuInterface){
       int [] blueCounter = tmuInterface.countBlueNodes();
       if(blueCounter!=null){           
           int max = blueCounter[0];
           for(int i=0; i<blueCounter.length; i++){
               if(blueCounter[i]>max)
                   max = blueCounter[i];
           }
           int playerBlueNodes = blueCounter[this.playerIndx-1];
           if(playerBlueNodes+1>=max)return true;
           else return false;
       }
       return true;
   }
   private Move attackConfidence(int round, double conf, TMUInterface tmuInterface, GMUInterface gmuInterface){
       //try to propose a move with a higher confidence than conf against the highest confidence leaf move
       Move move= null;
       Move moveToCounter=this.getHighestConfLeafMove(tmuInterface, gmuInterface);
       if(moveToCounter!=null)
           move = this.rmu.playProposeWithHigherConf(moveToCounter, conf,this.strategy, this.playerIndx);
       return move;
       
   }
   private Move getHighestConfLeafMove(TMUInterface tmuInterface, GMUInterface gmuInterface){
       Move moveToCounter = null;
       CompressedArgTreeNode highestConfLeaf = tmuInterface.highestConfLeaf(this.playerIndx);       
       if(highestConfLeaf!=null){
           moveToCounter = gmuInterface.treeNodeToMove(highestConfLeaf);          
       }
       return moveToCounter;
   }
   private Move attackWhenEverPossibleBuildBlind(int round, Move [] leafMoves, short gameFlag,short statusFlag){
       Move[] orderedLeafs = this.orderLeafMoves(leafMoves);
       boolean done=false;
       int i=0;
       Move move=null;
       while((!done)&&(i<orderedLeafs.length)){
           move = this.attackBuildOnly(leafMoves[i]);
           if(move!=null)   done=true;
           else             i++;           
       }
       if(!done){
         i=0;
         while((!done)&&(i<orderedLeafs.length)){
             move = this.attackDestroyOnly(leafMoves[i]);
             if(move!=null)   done=true;
             else             i++;           
         }         
       }
       return move;
   }
   private Move attackWhenEverPossibleBuildFocused(int round, Move [] leafMoves, short gameFlag,short statusFlag){
       Move[] directThreat = this.threatMoves(leafMoves);
       Move move = null;
       if(directThreat!=null){
           move = this.attackWhenEverPossibleBuildBlind(round,directThreat,gameFlag,statusFlag);
           if(move==null){
               Move[] otherMoves = this.otherMoves(leafMoves);
               if(otherMoves!=null)
                   move = this.attackWhenEverPossibleBuildBlind(round,otherMoves,gameFlag,statusFlag);
           }
       }
       else
           move = this.attackWhenEverPossibleBuildBlind(round,leafMoves,gameFlag,statusFlag);
       return move;
   }
   private Move attackWhenEverPossibleDestroyBlind(int round, Move [] leafMoves, short gameFlag,short statusFlag){
       Move[] orderedLeafs = this.orderLeafMoves(leafMoves);
       boolean done=false;
       int i=0;
       Move move=null;
       while((!done)&&(i<orderedLeafs.length)){
           move = this.attackDestroyOnly(leafMoves[i]);
           if(move!=null)   done=true;
           else             i++;           
       }
       if(!done){
         i=0;
         while((!done)&&(i<orderedLeafs.length)){
             move = this.attackBuildOnly(leafMoves[i]);
             if(move!=null)   done=true;
             else             i++;           
         }         
       }
       return move;
   }
   private Move attackWhenEverPossibleDestroyFocused(int round, Move [] leafMoves, short gameFlag,short statusFlag){
       Move move=null;
       Move[] directThreat = this.threatMoves(leafMoves);
       if(directThreat!=null){
           move = this.attackWhenEverPossibleDestroyBlind(round,directThreat,gameFlag,statusFlag);
           if(move==null){
               Move[] otherMoves = this.otherMoves(leafMoves);
               if(otherMoves!=null)
                   move = this.attackWhenEverPossibleDestroyBlind(round,otherMoves,gameFlag,statusFlag);
           }
       }
       else
           move = this.attackWhenEverPossibleDestroyBlind(round,leafMoves,gameFlag,statusFlag);
       return move;
   }

   /*2*/
   private Move attackWhenNeeded(int round, TMUInterface tmuInterface, GMUInterface gmuInterface, short gameFlag,short statusFlag){
       if((gameFlag>0)||(statusFlag>0))
           return this.attackWhenEverPossible(round,tmuInterface,gmuInterface,gameFlag, statusFlag);
       else{
           if(this.strategy.getSmartness()==2)
               return this.attackWhenNeededTree(round, tmuInterface, gmuInterface,gameFlag,statusFlag);       
           else{
               switch (this.strategy.getStrategyMode()){
                   case 1://build
                       return this.attackWhenNeededBuild(round,tmuInterface,gmuInterface,gameFlag, statusFlag);
                   default://destroy
                       return this.attackWhenNeededDestroy(round,tmuInterface,gmuInterface,gameFlag, statusFlag);                   
               }
           }
      }
   }   
   private Move attackWhenNeededBuild(int round, TMUInterface tmuInterface, GMUInterface gmuInterface, short gameFlag,short statusFlag){
       if(!tmuInterface.hasGreenNodes(this.playerIndx))
           return this.attackWhenEverPossibleBuild(round, tmuInterface, gmuInterface, gameFlag, statusFlag);       
       else
           return null;
   }
   private Move attackWhenNeededDestroyDominant(int round, TMUInterface tmuInterface, GMUInterface gmuInterface, short gameFlag,short statusFlag){
       int[] blueCounter = tmuInterface.countBlueNodes();
       if(blueCounter[this.playerIndx-1]==0)           
           return this.attackWhenEverPossibleDestroy(round, tmuInterface, gmuInterface, gameFlag, statusFlag);       
       else{           
           for(int i=1; i<blueCounter.length; i++){
               if(blueCounter[i]>blueCounter[this.playerIndx-1])
                   return this.attackWhenEverPossibleDestroy(round, tmuInterface, gmuInterface, gameFlag, statusFlag);
           }
           return null;
       }           
   }
   private Move attackWhenNeededDestroy(int round, TMUInterface tmuInterface, GMUInterface gmuInterface, short gameFlag,short statusFlag){
       int[] blueCounter = tmuInterface.countBlueNodes();
       if(blueCounter[this.playerIndx-1]==0)       
           return this.attackWhenEverPossibleDestroy(round, tmuInterface, gmuInterface, gameFlag, statusFlag);       
       else{
           for(int i=1; i<blueCounter.length; i++){
               if ((blueCounter[i]>0)&&(i!=(this.playerIndx-1)))
                  return this.attackWhenEverPossibleDestroy(round, tmuInterface, gmuInterface, gameFlag, statusFlag);               
           }
           return null;
       }           
   }
   /*the player need not to attack if it has the best position on the tree
     1) green confidence
     2) active green confidence
     3) dominant (all blue Nodes) so far*/
   private Move attackWhenNeededTree(int round, TMUInterface tmuInterface, GMUInterface gmuInterface, short gameFlag,short statusFlag){
       if(tmuInterface.greenConfStatus()){
           if(tmuInterface.greenConfPlayer()==this.playerIndx)
               return null;
       }
       if(tmuInterface.activeGreenConfStatus()){
           if(tmuInterface.activeGreenNodePlayer()==this.playerIndx)
               return null;
       }
       int[] blueCounter = tmuInterface.countBlueNodes();
       if(blueCounter[this.playerIndx-1]!=0){           
           for(int i=1; i<blueCounter.length; i++){
             if ((blueCounter[i]>0)&&(i!=(this.playerIndx-1))){
                 return this.attackWhenEverPossibleTree(round, tmuInterface,gmuInterface,gameFlag,statusFlag);
             }  
           }
           return null;
       }
       else{
           return this.attackWhenEverPossibleTree(round, tmuInterface,gmuInterface,gameFlag,statusFlag);
       }
   }   
   /*3*/
   private Move attackToPreventThreat(int round, TMUInterface tmuInterface, GMUInterface gmuInterface, short gameFlag,short statusFlag){
       /*1) try to secure the highest confidence position 
        by prposing a move with a higher confidence than green conf
        2) repeat for active green conf
        3) try to undermine the best position in the following order:
            1 - green conf
            2 - active green conf
        4) if the best position is secured (or failed to) then
        *   1 - try to counter direct threats
        *   2 - try to counter indirect threats
        5) if threats were countered or the player failed in counterin the,
            - - try to counter the dominant blue player(i.e. attack any blue leafs belonging to this player)*/
       
       Move move=null;
       /*1*/
       if(tmuInterface.greenConfStatus()){//green conf
           if(tmuInterface.greenConfPlayer()!= this.playerIndx){
               double greenConf = tmuInterface.greenConf();
               move = this.attackConfidence(round, greenConf, tmuInterface, gmuInterface);
               if(move!=null)
                   return move;
           }
       }
       
       /*2*/
       if(!tmuInterface.activeGreenEqualsGreenConf()){
           if(tmuInterface.activeGreenConfStatus()){//green conf 
               if(tmuInterface.activeGreenNodePlayer()!= this.playerIndx){
                   double activeGreenConf = tmuInterface.activeGreenConf();
                   move = this.attackConfidence(round, activeGreenConf, tmuInterface, gmuInterface);
                   if(move!=null)
                       return move;
               }
           }
        }
       
       /*3*/
        if(tmuInterface.greenConfStatus()){
           if(tmuInterface.greenConfPlayer()!= this.playerIndx){
               CompressedArgTreeNode[] greenConfLeafs = tmuInterface.greenConfCompressedLeafs();
               Move[] greenConfLeafsMoves= gmuInterface.leafsToMoves(greenConfLeafs);
               move=this.playLevel2(greenConfLeafsMoves);
               if(move!=null)
                   return move;
           }
        }
              
       /*4*/
       if(!tmuInterface.activeGreenEqualsGreenConf()){
           if(tmuInterface.activeGreenConfStatus()){
               if(tmuInterface.activeGreenNodePlayer()!= this.playerIndx){
                   CompressedArgTreeNode[] activeGreenConfLeafs = tmuInterface.activeGreenConfCompressedLeafs();
                   Move[] activeGreenConfLeafsMoves= gmuInterface.leafsToMoves(activeGreenConfLeafs);
                   move=this.playLevel2(activeGreenConfLeafsMoves);
                   if(move!=null)
                       return move;
               }
            }
       }
       
       /*5 direct threats (i.e. leafs that attacks the player directly)*/
       CompressedArgTreeNode[] compressedLeafs = tmuInterface.requestCompressedLeafs();
       Move[] leafMoves = gmuInterface.leafsToMoves(compressedLeafs);
       Move[] threat= this.threatMoves(leafMoves);
       if(threat!=null){
           if(this.strategy.getStrategyMode()==1)
               return this.attackWhenEverPossibleBuildFocused(round,threat,gameFlag,statusFlag);
           else
               return this.attackWhenEverPossibleDestroyFocused(round,threat,gameFlag,statusFlag); 
       }
       
       /*6*/
       CompressedArgTreeNode[] compressedIndirectThreat = tmuInterface.compressedIndirectThreats(this.playerIndx);
       Move[] indirectThreats = gmuInterface.leafsToMoves(compressedIndirectThreat);
       if(indirectThreats!=null){
           if(this.strategy.getStrategyMode()==1)
               return this.attackWhenEverPossibleBuildFocused(round,indirectThreats,gameFlag,statusFlag);
           else
               return this.attackWhenEverPossibleDestroyFocused(round,indirectThreats,gameFlag,statusFlag);            
       }
       
       /*7 - attack dominant blue player leafs*/
       int dominantBlue = tmuInterface.dominantBlue();
       if((dominantBlue!=-1)&&(dominantBlue!=this.playerIndx)){
           CompressedArgTreeNode[] compressedPlayerLeafs = tmuInterface.compressedPlayerLeafs(dominantBlue);
           Move[] domoinantBlueLeafs = gmuInterface.leafsToMoves(compressedPlayerLeafs);
           if(domoinantBlueLeafs!=null){
               if(this.strategy.getStrategyMode()==1)
                   return this.attackWhenEverPossibleBuildFocused(round,domoinantBlueLeafs,gameFlag,statusFlag);
               else
                   return this.attackWhenEverPossibleDestroyFocused(round,domoinantBlueLeafs,gameFlag,statusFlag);            
           }
       }
       return move;
   }

   /*------------------------------------------------------------------------------------------------------*/   
   
   /* ------ PLAY FUNCTIONS ------ */ 
   /**return a move responding to moveToAttack, in build mode*/
    protected Move attackBuild(Move moveToAttack){
        double moveToAttackConf = moveToAttack.getRule().getConfidence();
        if (strategy.getMinConf()<moveToAttackConf)
            strategy.setMinConf(moveToAttackConf);
        return this.rmu.attackBuild(moveToAttack, this.strategy, this.playerIndx);
    }
   /**return a move responding to moveToAttack, in destroy mode*/
   protected Move attackDestroy(Move moveToAttack){
       double moveToAttackConf = moveToAttack.getRule().getConfidence();
       if (strategy.getMinConf()<moveToAttackConf)
            strategy.setMinConf(moveToAttackConf);
       return this.rmu.attackDestroy(moveToAttack, this.strategy, this.playerIndx);
   }   
   /**return a move responding to moveToAttack, using only build moves*/
    protected Move attackBuildOnly(Move moveToAttack){
        double moveToAttackConf = moveToAttack.getRule().getConfidence();
        if (strategy.getMinConf()<moveToAttackConf)
            strategy.setMinConf(moveToAttackConf);
        return this.rmu.attackBuildOnly(moveToAttack, this.strategy, this.playerIndx);
    }
   /**return a move responding to moveToAttack, using only destroy moves*/
   protected Move attackDestroyOnly(Move moveToAttack){
       double moveToAttackConf = moveToAttack.getRule().getConfidence();
       if (strategy.getMinConf()<moveToAttackConf)
            strategy.setMinConf(moveToAttackConf);
       return this.rmu.attackDestroyOnly(moveToAttack, this.strategy, this.playerIndx);
   }
   //OUTPUT
    /**outputs the player basic information*/
    protected void output(){
        System.out.println("player Index: " + this.playerIndx);
        System.out.println("player Goal: " + this.rmu.getGoal());
        this.strategy.output();
    }
    /*-------------------------------------------------------------------------------*/

    //LEADERSHIP FUNCTIONS - DECISION MAKING UNIT
    //STRATEGY RANKING
    protected int strategyRank(){return this.strategy.strategyRank();}
    protected void setCounterGoals(short[] counterGoals) {this.rmu.setCounterGoals(counterGoals);}
    protected int getExperience(){return this.strategy.getExperience();}
    protected Move decide(Move[] groupMoves, int round, 
            TMUInterface tmuInterface,GMUInterface gmuInterface, short statusFlag, short gameFlag){
        if(this.leadershipUnit==null) return null;
        else return this.leadershipUnit.decide(groupMoves,round,tmuInterface,gmuInterface,statusFlag,gameFlag);
    }
    protected Move decideFirstMove(Move[] groupMoves){
        if(this.leadershipUnit==null) return null;
        else return this.leadershipUnit.decideFirstMove(groupMoves);
    }

    protected String outputParameters(){
        if(this.rmu==null) return null;
        String output= "dataFile: " + this.rmu.getDataFilePath() +"\n";
        output+="minimum support: " + this.rmu.getMinSupport() +"\n";
        output+=this.strategy.outputParameters();
        output+="-----------------------------------------------------\n";
        return output;
    }
    
}//end of the class