/*
 * HistoryLog.java
 *
 * Created on 02 September 2008, 14:14
 *
 * responsible for keeping track of the moves played in the game so far.
 * @author maya
 */

package GUIComponents;
//java packages
import java.io.*;
import java.util.*;

public class HistoryLog {    
    private String offlineHistoryLog = "c:\\output.txt";//the path of the offline history log
    /*the actual online log, 2d Array, arranged as follows:
     entry[0][0] is the first move, all other entries[a][a] are empty.
     entry[a][b] contains list of moves played by a against b*/
    private HistoryLogNode [][] historyArray;//
    
    //Constructors
    /**the default constructor*/
    public HistoryLog() {
        this.historyArray = null;
        this.offlineHistoryLog="c:\\output.txt";
    }
    /**constructs a new object of the class with the historyArray already defined*/
    public HistoryLog(int playersNo){
        this.historyArray = new HistoryLogNode[playersNo][playersNo];
        this.offlineHistoryLog="c:\\output.txt";
    }
    
    //Setters & Getters
    protected void setOfflineHistoryLog(String path){this.offlineHistoryLog=path;} 

    //methods
    //INSERT NEW MOVES
    /**insert the first move (fills in the [0][0] entry)*/
    protected void insertFirstMove(Move move){
        HistoryLogNode firstNode = new HistoryLogNode(move.getType(), 1, move.getRule());
        this.historyArray[0][0] = firstNode;
    }
    /**inserts move M played in round R, into the proper entry, 
     * if the move satisfies all the insetion 4 rules
     * returns 0 after inserting 
     * otherwise it reutrn the number of the first rule this move failed to satisfy*/
    protected int insertMove(Move move, int round){
        if(!rule4(move))    return 4;
        else{
            if(!rule1(move))    return 1;
            else{
                if(!rule2(move))    return 2;
                else{
                    if(!rule3(move))    return 3;
                    else{
                        //add node
                        insertClearedMove(move,round);
                        return 0;
                    }
                }
            }
        }        
    }
    /**inserts a cleared move, that has satisfied the 4 rules, into the proper entry list*/
    private void insertClearedMove(Move move, int round){
        int source = move.getSource()-1;
        int target = move.getTarget()-1;        
        if(this.historyArray[source][target]==null){            
            this.historyArray[source][target] = new HistoryLogNode(move.getType(), round, move.getRule());
        }
        else{
           HistoryLogNode  node =  this.historyArray[source][target];
           while(node.next()!=null)
               node = node.next();
           node.next(move.getType(), round, move.getRule());
        }
    }
    
    //CHECK FOR THE INSERTION RULES
    /**returns true if the source has not played the same move against the same player before*/
    private boolean rule1(Move move){        
        int source = move.getSource()-1;
        int target = move.getTarget()-1;
        if(this.historyArray[source][target]==null) return true;
        HistoryLogNode node = this.historyArray[source][target]; 
        while(node!=null){
            if(RuleNode.semiEquals(node.getRule(), move.getRule()))                
                return false;                
            else                
                node = node.next();  
        }
        return true;
    }    
    /**returns true if the source has not played the same move against another player before
     * if move is distinguish or unwanted consequences then no need to check     
     * if move is of Green Type then Check*/     
    private boolean rule2(Move move){        
        if((move.getType()==MoveType.DISTINGUISH)||(move.getType()==MoveType.UNWANTED_CONSEQUENCES))        
            return true;                
        int source = move.getSource()-1;
        int target = move.getTarget()-1;         
        boolean found = false;
        for(int i=0; i<this.historyArray.length; i++){
           if(i!=target){
               if(this.historyArray[source][i]!=null){
                   HistoryLogNode node = this.historyArray[source][i];
                   while(node!=null){
                      if(RuleNode.semiEquals(node.getRule(), move.getRule()))
                            return false;
                        else
                            node = node.next(); 
                   }
               }
           } 
        }
        return true;
    }
    /**returns true if no other player has played this move against the same target before*/
    private boolean rule3(Move move){
        int source = move.getSource()-1;
        int target = move.getTarget()-1;
        for(int i=0; i<this.historyArray.length; i++){
            if(i!=source){
                if(this.historyArray[i][target]!=null){
                    HistoryLogNode node = this.historyArray[i][target];
                    while(node!=null){
                        if(RuleNode.semiEquals(node.getRule(), move.getRule())){
                            switch (move.getType()){
                                case 1: case 4: case 5: case 6:
                                    if(move.getConfidence()>node.getRule().getConfidence())
                                        node = node.next();
                                    else
                                       return false;
                                    break;
                                case 2:
                                    if(move.getConfidence()<node.getRule().getConfidence())
                                        node = node.next();
                                    else
                                        return false;
                                    break;
                                default:
                                    node = node.next();
                                   // return false;                                    
                            }                            
                        }
                        else    node = node.next();
                    }
                }
            }
        }
        return true;
    }   
    /**returns false if the first player is repeating the first move*/
    private boolean rule4(Move move){
        if((move.getType()==1)||(move.getType()==4)){
            if(RuleNode.semiEquals(move.getRule(), historyArray[0][0].getRule()))
                return false;
            else
                return true;     
        }
        else return true;
    }
    
    //OUTPUT INTO THE OFFLINE LOG
    /**outputs the historylog contents to the screen*/
    protected void output(){
        //open the file
        FileWriter fileOutput;
        try {
            fileOutput= new FileWriter(offlineHistoryLog, true);
        }
        catch(IOException ioException) {
            System.out.println("Error writing to file: " + offlineHistoryLog + "\n");
            fileOutput=null;
        }
        
        if(this.historyArray[0][0]!=null){
            try{            
                fileOutput.write(historyArray[0][0].getString());                
                for(int j=1; j<this.historyArray.length; j++){
                    if (this.historyArray[0][j]!=null)
                    fileOutput.write(historyArray[0][j].getString());
                }
                for(int i=1; i<this.historyArray.length; i++){            
                    for(int j=0; j<this.historyArray.length; j++){
                        if (this.historyArray[i][j]!=null)
                        fileOutput.write(historyArray[i][j].getString());
                    }
                }
            }
            catch (IOException ioException){
                System.out.println("Error writing to File: " + this.offlineHistoryLog + "\n");	
            }

        }
        else
            System.out.println("historyLog is empty");
        //close file
        if(fileOutput!=null){
            try {
                      fileOutput.close();
             }
             catch (IOException ioException) {
                    System.out.println("Error closing file");
                    System.exit(1);
             }     
        }
    }
  
    //SEARCH METHODS
    /**returns the move played by source against target in the given round
     * if such move was not found it returns null*/
    protected Move findMove(int source, int target, int round, int leafIndx){
        Move move = null; 
        HistoryLogNode attackLine;        
        if(target ==0){
            attackLine = this.historyArray[0][0];
            if(source==1)
                move= new Move(source, 0, attackLine.getType(), attackLine.getRule(), leafIndx);
        }
        else{
           attackLine = this.historyArray[source-1][target-1];
           boolean found = false;
            while((attackLine!=null)&&(!found)){
                if(attackLine.getRound()==round){
                    move= new Move(source, target, attackLine.getType(), attackLine.getRule(),leafIndx);
                    found = true;
                }
                else
                   attackLine = attackLine.next(); 
            }
           
        }
        if(move==null)
            System.out.println("MOVE NOT FOUND IN HISTORY");
        return move;
    }
}//End of the Class
