/*
 * LeadershipUnit.java
 *
 * Created on 03 September 2008, 15:34
 * Operates the leader's responsibility and monitors the in-group discussions
 * @author maya
 */

package GUIComponents;

public class LeadershipUnit {
    int strategyRank;
    boolean type; //true = homo, false = hetero
    boolean supervisedLeadership; //true = supervised, false = unsupervised
    int playerIndx;
    boolean buildStrategy; //false = destroy;
    /** Creates a new instance of LeadershipUnit */
    public LeadershipUnit(int rank, int index){
        this.strategyRank= rank;
        this.playerIndx=index;
        switch(rank){
            case 1: case 3: case 5: case 7: case 9: case 11: case 13:
                this.buildStrategy=true;
                break;
            default:
                this.buildStrategy=false;
        }
    }
    public void setLeadershipType(boolean newType){this.type = newType;}
    public void setSupervision(boolean supervisionType){this.supervisedLeadership = supervisionType;}
    
    private boolean sameRuleSameOpp(Move[] groupMoves){
        Move move = groupMoves[0];
        int index=1;
        if(move ==null){
            while((index<groupMoves.length)&&(groupMoves[index]==null))
                index++;
            if(index<groupMoves.length)
                move = groupMoves[index];
        }
        if(index<groupMoves.length){
            while(index <groupMoves.length)
            for(int i=index; i<groupMoves.length; i++){
                if(groupMoves[i]!=null)
                    if (!Move.collides(move,groupMoves[i])) return false;                
            }
            return true;
        }
        else return true;        
    }    
    private boolean sameRuleDiffOpp(Move[] groupMoves){
        Move move = groupMoves[0];
        int index=1;
        if(move ==null){
            while((index<groupMoves.length)&&(groupMoves[index]==null))
                index++;
            if(index<groupMoves.length)
                move = groupMoves[index];
        }
        if(index<groupMoves.length){
            while(index <groupMoves.length)
            for(int i=index; i<groupMoves.length; i++){
                if(groupMoves[i]!=null)
                    if (!RuleNode.equals(groupMoves[i].getRule(), move.getRule())) return false;
            }
            return true;
        }
        else return true;        
    }    
    private boolean nullMoves(Move[] groupMoves){
        for(int i=0; i<groupMoves.length; i++)
            if(groupMoves[i]!=null) return false;
        return true;
    }
    private Move firstNotNull(Move[] groupMoves){        
        for(int i=0; i<groupMoves.length; i++)
            if(groupMoves[i]!=null) return groupMoves[i];
        return null;
    }

    //basics 
    private Move[] filterNull(Move[] moves){
        if (moves==null) return null;
        Move[] notNullMoves = null;
        for(int i=0; i<moves.length; i++){
            if(moves[i]!=null)
                notNullMoves = BasicOperations.append(notNullMoves,moves[i]);
        }
        return notNullMoves;
    }
    private Move[] orderMovesBuild(Move[] moves){
        if (moves==null) return null;
        if(moves.length==1) return moves;
        Move[] notNullMoves = this.filterNull(moves);
        if(notNullMoves==null) return null;
        for(int i=1; i<notNullMoves.length; i++){
            Move temp = notNullMoves[i];
            int j=i;
            while ((j > 0) && (notNullMoves[j-1].getConfidence()<temp.getConfidence())){
                notNullMoves[j] = notNullMoves[j-1];
                j--;
            }
            notNullMoves[j] = temp;
        }
        return notNullMoves;
   }
    private Move[] orderMovesDestroy(Move[] moves){
        if (moves==null) return null;
        if(moves.length==1) return moves;
        Move[] notNullMoves = this.filterNull(moves);
        if(notNullMoves==null) return null;
        for(int i=1; i<notNullMoves.length; i++){
            Move temp = notNullMoves[i];
            int j=i;
            while ((j > 0) && (notNullMoves[j-1].getConfidence()>temp.getConfidence())){
                notNullMoves[j] = notNullMoves[j-1];
                j--;
            }
            notNullMoves[j] = temp;
        }
        return notNullMoves;
   }
    private Move[] threatMoves(Move[] leafMoves){
       Move[] threatMoves = null;
       if(leafMoves!=null){
        for(int i=0; i<leafMoves.length; i++){
            if(leafMoves[i]!=null){
                if(leafMoves[i].getTarget()==this.playerIndx)
                    threatMoves = BasicOperations.append(threatMoves, leafMoves[i]);
            }
            else
            {
                int x;
            }
        }
       }
       return threatMoves;
   }
    private Move[] otherMoves(Move[] leafMoves){
       Move[] threatMoves = null;
       if(leafMoves!=null){
        for(int i=0; i< leafMoves.length; i++){
            if(leafMoves[i].getTarget()!=this.playerIndx)
                threatMoves = BasicOperations.append(threatMoves, leafMoves[i]);
        }
       }
       return threatMoves;
   }
    private Move[] movesAttacking(Move[] groupMoves, Move[] attackedMoves){        
        if(groupMoves==null) return null;
        Move[] movesAttacking  = null;
        for(int i=0; i<groupMoves.length; i++){
            Move move = this.moveAttacking(groupMoves[i], attackedMoves);
            if (move!=null)
                movesAttacking = BasicOperations.append(movesAttacking, move);
        }
        return movesAttacking;       
    }
    private Move moveAttacking(Move groupMove, Move[] attackedMoves){
        if(attackedMoves==null) return null;
        if(groupMove==null) return null;
        for(int i=0; i<attackedMoves.length; i++){
            if(groupMove.getTarget() == attackedMoves[i].getSource())
                return groupMove;
        }
        if(this.redirectable(groupMove)){
            Move bestTarget = this.bestTarget(groupMove, attackedMoves);
            if(bestTarget!=null){
                return this.redirect(groupMove, bestTarget);
            }
        }
        return null;
    }
    
    private Move bestTarget(Move groupMove, Move[] attackedMoves){
        if (attackedMoves==null) return null;
        if(attackedMoves.length==1){
            if(attackedMoves[0].getConfidence()<groupMove.getConfidence())
                return attackedMoves[0];
            else
                return null;
        }
        else{
            Move[] orderedAttackedMoves = this.orderMovesBuild(attackedMoves);
            for(int i=0; i<orderedAttackedMoves.length; i++){
                if(groupMove.getConfidence()>orderedAttackedMoves[i].getConfidence())
                    return orderedAttackedMoves[i];
            }
            return null;
        } 
    }
    //basic strategy blocks
    /*these methods specify the group Moves attacking the argumentation tree leafs 
     that should be attacked in each step of the higher level strategies*/    
    private Move[] higherThanGreenConf(Move[] groupMoves, TMUInterface tmuInterface){        
        double greenConf = tmuInterface.greenConf();
        Move [] higherThanGCMoves = null;
        if (groupMoves==null) return null;
        for(int i=0; i<groupMoves.length; i++){
            if((groupMoves[i] !=null)&&(groupMoves[i].getConfidence()>greenConf))
                higherThanGCMoves = BasicOperations.append(higherThanGCMoves, groupMoves[i]);
        }   
        return this.orderMovesBuild(higherThanGCMoves);         
    }
    private Move[] underminingGreenConf(Move[] groupMoves, TMUInterface tmuInterface,GMUInterface gmuInterface){        
        CompressedArgTreeNode[] greenNodeLeafs = tmuInterface.greenConfCompressedLeafs();
        if(greenNodeLeafs==null) return null;
        Move[] greenNodeLeafsMoves = gmuInterface.leafsToMoves(greenNodeLeafs);
        Move[] movesUnderminingGreenConf = this.movesAttacking(groupMoves, greenNodeLeafsMoves); 
        if(this.buildStrategy) return this.orderMovesBuild(movesUnderminingGreenConf);
        else    return this.orderMovesDestroy(movesUnderminingGreenConf);        
    }        
    private Move[] higherThanActiveGreenConf(Move[] groupMoves, TMUInterface tmuInterface){
        double activeGreenConf = tmuInterface.greenConf();
        Move [] higherThanAGCMoves = null;
        if (groupMoves==null) return null;
        for(int i=0; i<groupMoves.length; i++){
            if((groupMoves[i] !=null)&&(groupMoves[i].getConfidence()>activeGreenConf))
                higherThanAGCMoves = BasicOperations.append(higherThanAGCMoves, groupMoves[i]);
        }
        return this.orderMovesBuild(higherThanAGCMoves);
    }
    private Move[] underminingActiveGreenConf(Move[] groupMoves, TMUInterface tmuInterface,GMUInterface gmuInterface){
        CompressedArgTreeNode[] adtiveGreenNodeLeafs = tmuInterface.activeGreenConfCompressedLeafs();
        if(adtiveGreenNodeLeafs==null) return null;
        Move[] activeGreenNodeLeafsMoves = gmuInterface.leafsToMoves(adtiveGreenNodeLeafs);
        if(activeGreenNodeLeafsMoves==null) return null;
        Move[] movesUnderminingActiveGreenConf = this.movesAttacking(groupMoves, activeGreenNodeLeafsMoves); 
        if(this.buildStrategy) return this.orderMovesBuild(movesUnderminingActiveGreenConf);
        else    return this.orderMovesDestroy(movesUnderminingActiveGreenConf);        

    }        
    private boolean playBlue(TMUInterface tmuInterface){
       int [] blueCounter = tmuInterface.countBlueNodes();
       if(blueCounter!=null){           
           int max = blueCounter[0];
           for(int i=0; i<blueCounter.length; i++){
               if(blueCounter[i]>max)
                   max = blueCounter[i];
           }
           int playerBlueNodes = blueCounter[this.playerIndx];
           if(playerBlueNodes+1>=max)return true;
           else return false;
       }
       return true;    
    }
    private Move[] playBlueMoves(Move[] groupMoves, TMUInterface tmuInterface,GMUInterface gmuInterface){
        if(this.playBlue(tmuInterface)){
            Move[] groupBlueMoves = null;
            for(int i=0; i<groupMoves.length; i++){
                if((groupMoves[i]!=null)&&((groupMoves[i].getType()==2)||(groupMoves[i].getType()==3)))
                    groupBlueMoves = BasicOperations.append(groupBlueMoves, groupMoves[i]);
            }
            return this.orderMovesDestroy(groupBlueMoves);
        }
        else return null;        
    }    
    private Move[] attackingDirectThreat(Move[] groupMoves, TMUInterface tmuInterface,GMUInterface gmuInterface){
        CompressedArgTreeNode[] leafs = tmuInterface.requestCompressedLeafs();
        Move[] leafMoves = gmuInterface.leafsToMoves(leafs);
        Move [] directThreatMoves = this.threatMoves(leafMoves);
        if (directThreatMoves==null) return null;
        Move [] movesUnderminingDirectThreat = this.movesAttacking(groupMoves, directThreatMoves);
        if(this.buildStrategy)
            return this.orderMovesBuild(movesUnderminingDirectThreat);
        else return this.orderMovesDestroy(movesUnderminingDirectThreat);
    }    
    private Move[] attackingIndirectThreat(Move[] groupMoves, TMUInterface tmuInterface,GMUInterface gmuInterface){
        CompressedArgTreeNode[] indirectThreatleafs = tmuInterface.compressedIndirectThreats(this.playerIndx);
        Move[] indirectThreatleafsMoves = gmuInterface.leafsToMoves(indirectThreatleafs);        
        if (indirectThreatleafsMoves==null) return null; 
        Move [] movesUnderminingIndirectThreat = this.movesAttacking(groupMoves, indirectThreatleafsMoves);
        if(this.buildStrategy)
            return this.orderMovesBuild(movesUnderminingIndirectThreat);
        else return this.orderMovesDestroy(movesUnderminingIndirectThreat);
    }    
    private Move[] attackingDominantBluePlayer(Move[] groupMoves, TMUInterface tmuInterface,GMUInterface gmuInterface){
        int dominantBlue = tmuInterface.dominantBlue();
        CompressedArgTreeNode[] dominantBlueLeafs = tmuInterface.compressedPlayerLeafs(dominantBlue);
        Move[] dominantBlueLeafsMoves = gmuInterface.leafsToMoves(dominantBlueLeafs);        
        if (dominantBlueLeafsMoves==null) return null; 
        Move [] movesUnderminingBlueDom = this.movesAttacking(groupMoves, dominantBlueLeafsMoves);
        if(this.buildStrategy)
            return this.orderMovesBuild(movesUnderminingBlueDom);
        else return this.orderMovesDestroy(movesUnderminingBlueDom);
    }    
    //check group position (best or not) -- aditional
    
    //check Attack when needed conditions
    private boolean hasGreenNodes(TMUInterface tmuInterface){
        return tmuInterface.hasGreenNodes(this.playerIndx);
    }
    private boolean isBlueDominantMax(TMUInterface tmuInterface){
        int[] blueCounter = tmuInterface.countBlueNodes();
        if (blueCounter==null) return false;
        else{
            if(blueCounter[this.playerIndx]==0) return false;
            else{     
                for(int i=1; i<blueCounter.length; i++){
                    if(blueCounter[i]>blueCounter[this.playerIndx])
                    return false;
                }
                return true;
            }
        }
    }
    private boolean isBlueDominantAll(TMUInterface tmuInterface){
        int[] blueCounter = tmuInterface.countBlueNodes();
        if (blueCounter==null) return false;
        else{
            if(blueCounter[this.playerIndx]==0) return false;
            else{     
                for(int i=1; i<blueCounter.length; i++){
                    if((i!=(this.playerIndx))&&(blueCounter[i]!=0))
                        return false;
                }
                return true;
            }
        }
    }    
    private boolean isGreenConfDominante(TMUInterface tmuInterface){
        if((tmuInterface.greenConfStatus())&&(tmuInterface.greenConfPlayer()==this.playerIndx))
            return true;
        else return false;
    }
    private boolean isActiveGreenConfDominante(TMUInterface tmuInterface){
        if((tmuInterface.activeGreenConfStatus())&&(tmuInterface.activeGreenNodePlayer()==this.playerIndx))
            return true;
        else return false;
    }    
    
    //check group Moves against Leader Strategy
    /*each of these functions will return the group moves the follow the leader strategy ordered in the same way
      the strategy works*/
    private Move[] followAttackWhenPossibleTree(Move[] groupMoves, int round, 
            TMUInterface tmuInterface,GMUInterface gmuInterface){
        Move[] strategyMoves = null;
        if((tmuInterface.greenConfStatus())&&(tmuInterface.greenConfPlayer()!=this.playerIndx))
            strategyMoves = BasicOperations.append(strategyMoves,this.higherThanGreenConf(groupMoves,tmuInterface));
        if((tmuInterface.activeGreenConfStatus())&&(tmuInterface.activeGreenNodePlayer()!=this.playerIndx))
            strategyMoves = BasicOperations.append(strategyMoves,this.higherThanActiveGreenConf(groupMoves,tmuInterface));
        if(this.playBlue(tmuInterface))
            strategyMoves = BasicOperations.append(strategyMoves,this.playBlueMoves(groupMoves,tmuInterface,gmuInterface));
        strategyMoves = BasicOperations.append(strategyMoves,this.bestMoveFocused(groupMoves,round,tmuInterface,gmuInterface));
        return strategyMoves;      
    }

    private Move[] followAttackWhenNeededTree( Move[] groupMoves, int round, 
            TMUInterface tmuInterface,GMUInterface gmuInterface){        
        if((!this.isGreenConfDominante(tmuInterface))&&
                (!this.isActiveGreenConfDominante(tmuInterface))&&
                (!this.isBlueDominantAll(tmuInterface)))
            return this.followAttackWhenPossibleTree(groupMoves,round,tmuInterface,gmuInterface);
        else
            return null;
    }
    private Move[] followAttackToPreventThreat(Move[] groupMoves, int round, 
            TMUInterface tmuInterface,GMUInterface gmuInterface){
        if(this.buildStrategy)
            return this.followAttackToPreventThreatBuild(groupMoves, round, tmuInterface, gmuInterface);
        else return this.followAttackToPreventThreatDestroy(groupMoves, round, tmuInterface, gmuInterface);
    }
    private Move[] followAttackToPreventThreatBuild(Move[] groupMoves, int round, 
            TMUInterface tmuInterface,GMUInterface gmuInterface){
        Move[] strategyMoves = null;
        if((tmuInterface.greenConfStatus())&&(tmuInterface.greenConfPlayer()!=this.playerIndx))
            strategyMoves = BasicOperations.append(strategyMoves,this.higherThanGreenConf(groupMoves,tmuInterface));
        if((tmuInterface.activeGreenConfStatus())&&(tmuInterface.activeGreenNodePlayer()!=this.playerIndx))
            strategyMoves = BasicOperations.append(strategyMoves,this.higherThanActiveGreenConf(groupMoves,tmuInterface));
        if((tmuInterface.greenConfStatus())&&(tmuInterface.greenConfPlayer()!=this.playerIndx))
            strategyMoves = BasicOperations.append(strategyMoves,this.underminingGreenConf(groupMoves,tmuInterface,gmuInterface));
        if((tmuInterface.activeGreenConfStatus())&&(tmuInterface.activeGreenNodePlayer()!=this.playerIndx))
            strategyMoves = BasicOperations.append(strategyMoves,this.underminingActiveGreenConf(groupMoves,tmuInterface,gmuInterface));
        strategyMoves = BasicOperations.append(strategyMoves,this.attackingDirectThreat(groupMoves,tmuInterface,gmuInterface));
        strategyMoves = BasicOperations.append(strategyMoves,this.attackingIndirectThreat(groupMoves,tmuInterface,gmuInterface));
        strategyMoves = BasicOperations.append(strategyMoves,this.attackingDominantBluePlayer(groupMoves,tmuInterface,gmuInterface));
        return strategyMoves;      
    }
    private Move[] followAttackToPreventThreatDestroy(Move[] groupMoves, int round, 
            TMUInterface tmuInterface,GMUInterface gmuInterface){
        Move[] strategyMoves = null;
        if((tmuInterface.greenConfStatus())&&(tmuInterface.greenConfPlayer()!=this.playerIndx)){
            strategyMoves = BasicOperations.append(strategyMoves,this.underminingGreenConf(groupMoves,tmuInterface,gmuInterface));
            strategyMoves = BasicOperations.append(strategyMoves,this.higherThanGreenConf(groupMoves,tmuInterface));
        }
        if((tmuInterface.activeGreenConfStatus())&&(tmuInterface.activeGreenNodePlayer()!=this.playerIndx)){
            strategyMoves = BasicOperations.append(strategyMoves,this.higherThanActiveGreenConf(groupMoves,tmuInterface));            
            strategyMoves = BasicOperations.append(strategyMoves,this.underminingActiveGreenConf(groupMoves,tmuInterface,gmuInterface));            
        }
        strategyMoves = BasicOperations.append(strategyMoves,this.attackingDirectThreat(groupMoves,tmuInterface,gmuInterface));
        strategyMoves = BasicOperations.append(strategyMoves,this.attackingIndirectThreat(groupMoves,tmuInterface,gmuInterface));
        strategyMoves = BasicOperations.append(strategyMoves,this.attackingDominantBluePlayer(groupMoves,tmuInterface,gmuInterface));
        return strategyMoves;
    }
    private Move bestMoveFollowStrategy(Move[] strategyMoves){
        if (strategyMoves==null) return null;
        else return strategyMoves[0];
    }
    private Move bestMoveBlind(Move[] groupMoves, int round, 
            TMUInterface tmuInterface,GMUInterface gmuInterface){
        if(this.buildStrategy){
            Move[] orderedMoves = this.orderMovesBuild(groupMoves);
            if(orderedMoves==null) return null;
            else    return orderedMoves[0];
        }
        else{
            Move[] orderedMoves = this.orderMovesDestroy(groupMoves);
            if(orderedMoves==null) return null;
            else    return orderedMoves[0];
        }
    }
    private Move bestMoveFocused(Move[] groupMoves, int round, 
            TMUInterface tmuInterface,GMUInterface gmuInterface){
        CompressedArgTreeNode[] compressedLeafs = tmuInterface.requestCompressedLeafs();
        Move[] leafMoves = gmuInterface.leafsToMoves(compressedLeafs);            
        Move[] threatMoves = this.threatMoves(leafMoves);
        if(threatMoves!=null){
              Move move = this.bestMoveBlind(groupMoves, round, tmuInterface, gmuInterface);
              if(move == null)
                  return this.bestMoveBlind(groupMoves, round, tmuInterface, gmuInterface);
              else
                  return move;
        }
        else return this.bestMoveBlind(groupMoves, round, tmuInterface, gmuInterface);
    }
    private Move redirect(Move moveTobeRiderected, Move attackedMove){
        Move move;
        switch(moveTobeRiderected.getType()){            
            case 1: case 4:
                //(int source, int target, int type, RuleNode rule, int Attackedleaf)
                move = new Move(moveTobeRiderected.getSource(), attackedMove.getSource(),
                        moveTobeRiderected.getType(), moveTobeRiderected.getRule(), moveTobeRiderected.getLeafIndx());
                return move;
            case 5: case 6:
                move = new Move(moveTobeRiderected.getSource(), attackedMove.getSource(),
                        1, moveTobeRiderected.getRule(), moveTobeRiderected.getLeafIndx());
                return move;                
            default:
                return null;
        }
    }
    private boolean redirectable(Move move){
        switch(move.getType()){
            case 1: case 4: case 5: case 6: return true;
            default:    return false;
        }
    }
    
    public Move decide(Move[] groupMoves, int round, 
            TMUInterface tmuInterface,GMUInterface gmuInterface, short statusFlag, short gameFlag){
        if(groupMoves==null) return null;
        Move[] notNullMoves = this.filterNull(groupMoves);
        if(notNullMoves==null) return null;
        if(notNullMoves.length==1) return notNullMoves[0];
        switch(this.strategyRank){
            case 1: //Prevent Threat + Build
                return this.bestMoveFollowStrategy(this.followAttackToPreventThreatBuild(notNullMoves,round,tmuInterface,gmuInterface));
            case 2: //Prevent Threat + Destroy
                return this.bestMoveFollowStrategy(this.followAttackToPreventThreatDestroy(notNullMoves,round,tmuInterface,gmuInterface));
            case 3: case 4://Attack when needed tree
                return this.bestMoveFollowStrategy(this.followAttackWhenNeededTree(notNullMoves,round,tmuInterface,gmuInterface));
            case 5:case 6:////Attack when possible tree
                return this.bestMoveFollowStrategy(this.followAttackWhenPossibleTree(notNullMoves,round,tmuInterface,gmuInterface));
            case 7://Attack when needed Focused + Build
                if((statusFlag==2)||(gameFlag==2)) return this.bestMoveFocused(notNullMoves,round,tmuInterface,gmuInterface);
                if(this.hasGreenNodes(tmuInterface)) return null;
                else return this.bestMoveFocused(notNullMoves,round,tmuInterface,gmuInterface);                
            case 8://Attack when needed Focused + Destroy
                if((statusFlag==2)||(gameFlag==2)) return this.bestMoveFocused(notNullMoves,round,tmuInterface,gmuInterface);
                if(this.isBlueDominantMax(tmuInterface)) return null;
                else return this.bestMoveFocused(notNullMoves,round,tmuInterface,gmuInterface);
            case 9:case 10://Attack when possible Focused
                return this.bestMoveFocused(notNullMoves,round,tmuInterface,gmuInterface);
            case 11://Attack when needed Blind + Build
                if((statusFlag==2)||(gameFlag==2)) return this.bestMoveBlind(notNullMoves,round,tmuInterface,gmuInterface);
                if(this.hasGreenNodes(tmuInterface)) return null;
                else return this.bestMoveBlind(notNullMoves,round,tmuInterface,gmuInterface);
            case 12://Attack when needed Blind + Destroy
                if((statusFlag==2)||(gameFlag==2)) return this.bestMoveBlind(notNullMoves,round,tmuInterface,gmuInterface);
                if(this.isBlueDominantMax(tmuInterface)) return null;
                else return this.bestMoveBlind(notNullMoves,round,tmuInterface,gmuInterface);
            case 13:case 14: //Attack when possible Blind
                return this.bestMoveBlind(notNullMoves,round,tmuInterface,gmuInterface);
        }
        return null;
    }
    public Move decideFirstMove(Move[] firstMoves){
        if (firstMoves==null) return null;
        Move[] notNullMoves = this.filterNull(firstMoves);
        if (notNullMoves==null) return null;
        if(notNullMoves.length==1) return notNullMoves[0];
        Move[] orderedMoves = this.orderMovesBuild(notNullMoves);
        if(orderedMoves==null) return null;
        else  return orderedMoves[0];
    }    
    
    /*if group type is homogeneous then there's no need to check if moves are following the leader strategy???*/
    public Move decideHomogeneous(Move[] groupMoves, int round, 
            TMUInterface tmuInterface,GMUInterface gmuInterface, short statusFlag, short gameFlag){
       return null; 
    }

}

