/* Class structure

AssocRuleMining
      |
      +-- RuleList
	    |
      	    +-- RuleNode*/

/** encapsulates the basic Asociation Rules (ARs)
@author Frans Coenen
@ modified by Maya Wardeh for PADUA system 2007
@ modified by Maya Wardeh for PISA  system 2008*/

package GUIComponents;
// Java packages
import java.io.*;
import java.util.*;

public class RuleNode {    	
    protected short[] antecedent; //Antecedent of AR.	
    protected short[] consequent;//Consequent of AR.	
    protected double confidenceForRule=0.0;//The confidence associate with the rule node	
    protected RuleNode next = null; //Link to next nod
    protected boolean isfirst=true;   //true if the node is the first in the list
    
    /** Three argument constructor
	@param antecedent the antecedent (LHS) of the AR.
    	@param consequent the consequent (RHS) of the AR.
    	@param support the associated confidence value. */
    public RuleNode(short[] ante, short[]cons, double confValue) {
            this.antecedent        = ante;
	    this.consequent        = cons;
	    this.confidenceForRule = confValue;
        }
        
    /*Setters and Getters for private members*/
    public short[] getAntecednt(){return this.antecedent;}
    public short[] getConsequent(){return this.consequent;}
    public double getConfidence(){return this.confidenceForRule;}
    public void setAntecednt(short[] newAnt){
		AssocRuleMining arm = new AssocRuleMining();
		this.antecedent = arm.copyItemSet(newAnt);
        }
    public void setConsequent(short[] newCon){
		AssocRuleMining arm = new AssocRuleMining();
		this.consequent = arm.copyItemSet(newCon);
            } 
    public  void setConfidence(double newConf){this.confidenceForRule = newConf;}

    /*Methods*/
    /**outputs the AR to the screen*/
    public String output(){
        return this.getString();
    }
        
    /**converts the AR to a string */
    protected String itemSetToString(short[] itemSet) {	
        //Loop through item set elements
        String itemSetString="";
        if (itemSet == null) itemSetString ="item set is empty";
        else {
            int counter = 0;
            for (int index=0;index<itemSet.length;index++) {
                if (counter == 0) {
                    counter++;
                    itemSetString = itemSetString +" {";
                }
                else itemSetString = itemSetString +" ";
                itemSetString = itemSetString + String.valueOf(itemSet[index]);
            }
            itemSetString = itemSetString +"} ";
        }
        return itemSetString;
    }
    /**returns the string representing the rule*/
    public String getString(){
        String ruleString = this.itemSetToString(this.antecedent); 
        ruleString = ruleString + " -> ";
        ruleString = ruleString +this.itemSetToString(this.consequent);
        ruleString = ruleString +"\t\t";
        ruleString = ruleString + String.valueOf(this.confidenceForRule);
        ruleString = ruleString +"\n";
        return ruleString;
    }
 
    /*EQUAL operations*/
    /**returns true if the two rules (rule1, rule2) are exactly the same*/
    public static boolean equals(RuleNode rule1, RuleNode rule2) {
        if(rule1.getConfidence()==rule2.getConfidence()){          
            short [] ant1 = rule1.getAntecednt();
            short [] ant2 = rule2.getAntecednt();
            if (isEqual(ant1,ant2))
            {
                short [] con1 = rule1.getConsequent();
                short [] con2 = rule2.getConsequent();
                if (isEqual(con1,con2)) return true;
                else return false;
            }
            else return false;
        }
        else return false;
    }    
    /**returns true if the two rules (rule1, rule2) have the same antecedents and cosequents
     * regardless the confidence*/
    public static boolean semiEquals(RuleNode rule1, RuleNode rule2) {
        short [] ant1 = rule1.getAntecednt();
        short [] ant2 = rule2.getAntecednt();
        if (isEqual(ant1,ant2)){
            short [] con1 = rule1.getConsequent();
            short [] con2 = rule2.getConsequent();
            if (Arrays.equals(con1,con2)) return true;
	    else return false;
        }
        else return false;       
    }  
    /**returns true if the current rule semi equals 'rule' 
     * but has better confidenct that 'rule'*/
    public boolean betterthan(RuleNode rule) {
        if (RuleNode.semiEquals(this,rule)){
            if(this.confidenceForRule>rule.getConfidence())
                return true;
            else
                return false;
        }
        else return false;    
    }
    /**returns true if itemSet1 and itemSet2 are equal*/    
    private static boolean isEqual(short[] itemSet1, short[] itemSet2) {
            //If no itemSet2 (i.e. itemSet2 is null return false)
            if (itemSet2 == null) return(false);
            // Compare sizes, if not same length they cannot be equal.
            int length1 = itemSet1.length;
            int length2 = itemSet2.length;
            if (length1 != length2) return(false);
            // Same size compare elements
            Arrays.sort(itemSet1);
            Arrays.sort(itemSet2);
            for (int index=0;index < length1;index++){
                if (itemSet1[index] != itemSet2[index]) return(false);
            }
            // itemSet the same.
            return(true);
        }	
}//End of the Class
