/*
 * RulesMiningUnit.java
 *
 * Created on 03 September 2008, 16:40
 *
 *  used by GroupPlayer.java to produce the ARs needed in each round of the game
 *
 *  @author maya
 */

package GUIComponents;

//java packages
import java.util.*;

public class RulesMiningUnit {
    private String dataFilePath;//the path to the player's dataset
    private short goal; //the class that this player tries to prove true
    private short[] counterGoals;//the classes that this player tries to prove false
    private short[] currentCase;//the current case under argumentation
    private PartialSupportTree pTree; //the p-tree representing the player's data
    private double min_support; //The minimum support threshold of the game    
    private RuleList playedRules=null;//the moves this player played so far
    /** Creates a new instance of RulesMiningUnit */
    public RulesMiningUnit(String dataFile, short goal, short[] counterGoals, double minsupport) {
        this.dataFilePath = dataFile;
        this.goal = goal;
        this.counterGoals= counterGoals;
        this.min_support = minsupport;
    }
    
    //setters and getters
    protected short getGoal(){return this.goal;}
    protected short[] getCounterGoals(){return this.counterGoals;}
    protected String getDataFilePath(){return this.dataFilePath;}
    protected double getMinSupport(){return this.min_support;}
    protected void setCurrentCase(short[] inputCase){
        currentCase = new short[inputCase.length];
	for(int i=0; i<inputCase.length; i++)
		currentCase[i] = inputCase[i];
    }  
    protected void setCounterGoals(short[] newCounterGoals){
        this.counterGoals = new short[newCounterGoals.length];
	for(int i=0; i<newCounterGoals.length; i++)
		counterGoals[i] = newCounterGoals[i];
    }  
   //methods
   //BASICS
   /*create the P-Tree that represents the data stored in the dataFile*/    
    protected int GetData(){
	String[] args = {"-F"+this.dataFilePath, "-S"+this.min_support ,"-C"+40.0};
	pTree = new PartialSupportTree(args);
	int numberOfRows = pTree.inputDataSet();        
	pTree.createPtree();
        return numberOfRows;
    }   
    /*create the T-Tree of the instance P-Tree*/
    protected int buildTTree(){
	int maxLevel =  pTree.createTotalSupportTreeINT();	
	return maxLevel;
    }      
    /**initializes the rule mining unit*/
    protected void initialize(short[] currentCase){
       this.setCurrentCase(currentCase);
       /*this.GetData();
       this.buildTTree();*/
   }
    /**insert a new rule into the set of playedRules*/
    protected void insertRuleintoRulelist(RuleNode rule){
       if (this.playedRules==null)
           playedRules = new RuleList();
       playedRules.insertRuleintoRulelist(rule);
    } 
    /*-------------------------------------------------------------------------------*/ 

   /*-------------------------------------------------------------------------------*/ 
      /* ------ PLAY FUNCTIONS ------ */   
    /**return a move responding to moveToAttack, in destroy mode*/
    protected Move attackDestroy(Move moveToAttack, Strategy strategy, int playerIndx){
      //simplest plan-- to be replaced with overall strategy
      Move move = null;
      double moveToAttackConf = moveToAttack.getRule().getConfidence();
      if (strategy.getMinConf()<moveToAttackConf)
          strategy.setMinConf(moveToAttackConf);
      switch (moveToAttack.getType())
      {
         case 1://propose rule
         if(moveToAttack.getSource()!= playerIndx){
             strategy.setDefendingStatus(false);             
              move = this.playUnwnatedConsequences(moveToAttack,strategy,playerIndx);
              if (move == null){
                   move = this.playDistinguish(moveToAttack,strategy,playerIndx); 
                   if (move == null)
                   {
                     move = this.playCounterRule(moveToAttack,strategy,playerIndx);	
                     if (move != null)// counter rule ok
                         this.insertRuleintoRulelist(move.getRule());
                   }
                   else //distinguish ok
                     this.insertRuleintoRulelist(move.getRule());
                 }
                 else //unwanted ok
                   this.insertRuleintoRulelist(move.getRule());             
         }
	 break;
	/**********************************************************************/
	case 2://distinguish
            if(moveToAttack.getTarget()== playerIndx){
                 move = this.playUnwnatedConsequences(moveToAttack,strategy,playerIndx);
                 if (move == null){
                     move = this.playIncreaseConfidence(moveToAttack,strategy,playerIndx);
                     if(move==null){
                         if(strategy.getDefendingStatus()){
                             double confidence = moveToAttack.getRule().getConfidence();
                             move = this.playPropose(moveToAttack,strategy,playerIndx);
                         }
                         else
                             move = this.playCounterRule(moveToAttack,strategy,playerIndx);	
                         if (move != null)// counter rule ok
                             this.insertRuleintoRulelist(move.getRule());                         
                     }
                     else//increase OK
                         this.insertRuleintoRulelist(move.getRule());
                 }
                 else//unwanted OK
                     this.insertRuleintoRulelist(move.getRule());
            }
            break;
	/**********************************************************************/
	case 3://unwanted consequences
         if(moveToAttack.getTarget()== playerIndx){
             if(moveToAttack.getConfidence()<=strategy.getDistThreshold())
                 strategy.setDefendingStatus(false); 
             move = this.playWithdraw(moveToAttack,strategy,playerIndx);
             if (move == null)
             {           
               move = this.playPropose(moveToAttack,strategy,playerIndx);
               if (move != null)//propose ok
                   this.insertRuleintoRulelist(move.getRule());
             }
             else //withdraw ok
               this.insertRuleintoRulelist(move.getRule());
         }
	 break;
	/**********************************************************************/
	case 4://counter rule
         if(moveToAttack.getSource()!= playerIndx){
             strategy.setDefendingStatus(false);
             move = this.playUnwnatedConsequences(moveToAttack,strategy,playerIndx);
             if (move == null)
             {
               move = this.playDistinguish(moveToAttack,strategy,playerIndx); 
               if (move == null)
               {
                 move = this.playPropose(moveToAttack,strategy,playerIndx);
                 if (move != null)// propose ok
                     this.insertRuleintoRulelist(move.getRule());
               }
               else //distinguish ok
                 this.insertRuleintoRulelist(move.getRule());
             }
             else //unwanted ok
               this.insertRuleintoRulelist(move.getRule());
         }
	 break;
	/**********************************************************************/
  	case 5://increase confidence	 
         if(moveToAttack.getSource()!= playerIndx){
             move = this.playUnwnatedConsequences(moveToAttack,strategy,playerIndx);
             if (move == null){
               move = this.playDistinguish(moveToAttack,strategy,playerIndx); 
               if (move == null)
               {
                 move = this.playCounterRule(moveToAttack,strategy,playerIndx);	
                 if (move != null)// counter rule ok
                     this.insertRuleintoRulelist(move.getRule());
               }
               else //distinguish ok
                 this.insertRuleintoRulelist(move.getRule());
             }
             else //unwanted ok
               this.insertRuleintoRulelist(move.getRule());
         }
	 break;
	/**********************************************************************/
  	case 6://withdraw unwanted consequences
         if(moveToAttack.getConfidence()<=strategy.getDistThreshold())
             strategy.setDefendingStatus(false);                      
         if(moveToAttack.getSource()!= playerIndx){
             move = this.playUnwnatedConsequences(moveToAttack,strategy,playerIndx);
             if (move == null)
             {
               move = this.playDistinguish(moveToAttack,strategy,playerIndx); 
               if (move == null)
               {
                 move = this.playCounterRule(moveToAttack,strategy,playerIndx);	
                 if (move != null)// counter rule ok
                     this.insertRuleintoRulelist(move.getRule());
               }
               else //distinguish ok
                 this.insertRuleintoRulelist(move.getRule());
             }
             else //unwanted ok
               this.insertRuleintoRulelist(move.getRule());
         }
	 break;
	/**********************************************************************/
      }
      return move;	
    }
    /**return a move responding to moveToAttack, in build mode*/
    protected Move attackBuild(Move moveToAttack, Strategy strategy, int playerIndx){
      //simplest plan-- to be replaced with overall strategy
      Move move = null;
      double moveToAttackConf = moveToAttack.getRule().getConfidence();
      switch (moveToAttack.getType()){
         case 1://propose rule
             if(moveToAttack.getSource()!= playerIndx){
                 strategy.setDefendingStatus(false);
                 move = this.playCounterRule(moveToAttack,strategy,playerIndx);
                 if(move==null){
                     move = this.playUnwnatedConsequences(moveToAttack,strategy,playerIndx);
                     if(move==null){
                         move = this.playDistinguish(moveToAttack,strategy,playerIndx);
                         if (move!=null)//distinguish OK
                             this.insertRuleintoRulelist(move.getRule());
                     }
                     else//unwanted OK
                         this.insertRuleintoRulelist(move.getRule());
                 }
                 else//counter attack ok 
                     this.insertRuleintoRulelist(move.getRule());
             }
	 break;
	/**********************************************************************/
	case 2://distinguish
         if(moveToAttack.getTarget()== playerIndx){
             strategy.setDefendingStatus(true); 
             move = this.playPropose(moveToAttack,strategy,playerIndx);
             if(move==null){
                 move = this.playUnwnatedConsequences(moveToAttack,strategy,playerIndx);
                 if(move == null){
                     move = this.playIncreaseConfidence(moveToAttack,strategy,playerIndx);
                     if(move !=null)//increase OK
                         this.insertRuleintoRulelist(move.getRule());
                 }
                 else//Unwanted OK
                     this.insertRuleintoRulelist(move.getRule());  
             }
             else//propose OK
               this.insertRuleintoRulelist(move.getRule());  
         }
	 break;
	/**********************************************************************/
	case 3://unwanted consequences
            if(moveToAttack.getTarget()== playerIndx){
                if(moveToAttack.getConfidence()<=strategy.getDistThreshold())
                    strategy.setDefendingStatus(false);
                else
                    strategy.setDefendingStatus(true);
                move = this.playPropose(moveToAttack,strategy,playerIndx);
                if(move == null){
                    move = this.playWithdraw(moveToAttack,strategy,playerIndx);
                    if(move!=null)
                        this.insertRuleintoRulelist(move.getRule());
                }
                else //propose OK
                    this.insertRuleintoRulelist(move.getRule());
            }
	 break;
	/**********************************************************************/
	case 4://counter rule
            if(moveToAttack.getSource()!= playerIndx){
                strategy.setDefendingStatus(false);
                move = this.playPropose(moveToAttack,strategy,playerIndx);
                if(move==null){
                    move = this.playUnwnatedConsequences(moveToAttack,strategy,playerIndx);
                    if(move==null){
                        move = this.playDistinguish(moveToAttack,strategy,playerIndx);
                        if(move!=null)//distinguish OK
                            this.insertRuleintoRulelist(move.getRule());
                    }
                    else//unwanted OK
                        this.insertRuleintoRulelist(move.getRule());
                }
                else//propose OK
                    this.insertRuleintoRulelist(move.getRule());
            }
	 break;
	/**********************************************************************/
  	case 5://increase confidence	 
            if(moveToAttack.getSource()!= playerIndx){
                move = this.playCounterRule(moveToAttack,strategy,playerIndx);
                if(move==null){
                    move = this.playUnwnatedConsequences(moveToAttack,strategy,playerIndx);
                    if(move==null){
                        move = this.playDistinguish(moveToAttack,strategy,playerIndx);
                        if(move!=null)//distinguish OK
                            this.insertRuleintoRulelist(move.getRule());
                    }
                    else// unwatned OK
                        this.insertRuleintoRulelist(move.getRule());
                }
                else // counter OK
                    this.insertRuleintoRulelist(move.getRule());
            }
	 break;
	/**********************************************************************/
  	case 6://withdraw unwanted consequences            
            if(moveToAttack.getSource()!= playerIndx){
                if(moveToAttack.getConfidence()<=strategy.getDistThreshold())
                    strategy.setDefendingStatus(false);
                else
                    strategy.setDefendingStatus(true);
                move = this.playCounterRule(moveToAttack,strategy,playerIndx);
                if(move==null){
                    move = this.playUnwnatedConsequences(moveToAttack,strategy,playerIndx);
                    if(move==null){
                        move = this.playDistinguish(moveToAttack,strategy,playerIndx); 
                        if(move!=null)//distinguish OK
                            this.insertRuleintoRulelist(move.getRule());
                    }
                    else//unwanted OK
                        this.insertRuleintoRulelist(move.getRule());
                }
                else //counter OK
                    this.insertRuleintoRulelist(move.getRule());
            }
            break;
	/**********************************************************************/
      }
      return move;	
    }
    /**return a move responding to moveToAttack, using only build moves*/
    protected Move attackBuildOnly(Move moveToAttack, Strategy strategy, int playerIndx){
        Move move = null;
        double moveToAttackConf = moveToAttack.getRule().getConfidence();
        switch (moveToAttack.getType()){
          case 1://propose rule
              if(moveToAttack.getSource()!= playerIndx){
                  strategy.setDefendingStatus(false);
                  move = this.playCounterRule(moveToAttack, strategy, playerIndx);
                  if (move!=null)//Counter Rule OK
                      this.insertRuleintoRulelist(move.getRule());
              }
              break;
            case 2://distinguish
                if(moveToAttack.getTarget()== playerIndx){
                    strategy.setDefendingStatus(true);
                    move = this.playPropose(moveToAttack,strategy, playerIndx);
                    if(move==null){
                        move = this.playIncreaseConfidence(moveToAttack,strategy, playerIndx);
                        if(move !=null)//increase OK
                            this.insertRuleintoRulelist(move.getRule());
                    }
                    else//propose ok
                        this.insertRuleintoRulelist(move.getRule());
                }
                break;
            case 3://unwanted consequences
                if(moveToAttack.getTarget()== playerIndx){
                    if(moveToAttack.getConfidence()<=strategy.getDistThreshold())
                        strategy.setDefendingStatus(false);
                    else
                        strategy.setDefendingStatus(true);
                    
                    move = this.playPropose(moveToAttack,strategy, playerIndx);
                    if(move==null){
                        move = this.playWithdraw(moveToAttack,strategy, playerIndx);
                        if(move !=null)//withdraw OK
                            this.insertRuleintoRulelist(move.getRule());
                    }
                    else//propose ok
                        this.insertRuleintoRulelist(move.getRule());
                }
                break;
            case 4://counter rule
                if(moveToAttack.getSource()!= playerIndx){
                    strategy.setDefendingStatus(false);
                    move = this.playPropose(moveToAttack,strategy, playerIndx);
                    if(move!=null)//propose OK
                        this.insertRuleintoRulelist(move.getRule());
                }
                break;
            case 5://increase confidence
                if(moveToAttack.getSource()!= playerIndx){
                    move = this.playCounterRule(moveToAttack,strategy, playerIndx);
                    if(move!=null)//counter OK
                        this.insertRuleintoRulelist(move.getRule());
                }
                break;
            case 6://withdraw unwanted consequences
                if(moveToAttack.getSource()!= playerIndx){
                    if(moveToAttack.getConfidence()<=strategy.getDistThreshold())
                        strategy.setDefendingStatus(false);
                    else
                        strategy.setDefendingStatus(true);
                    move = this.playCounterRule(moveToAttack,strategy, playerIndx);
                    if(move!=null)//counter OK
                        this.insertRuleintoRulelist(move.getRule());
                }
                break;
        }
        return move;
    }
    /**return a move responding to moveToAttack, using only destroy moves*/
    protected Move attackDestroyOnly(Move moveToAttack, Strategy strategy, int playerIndx){
        Move move = null;
        double moveToAttackConf = moveToAttack.getRule().getConfidence();
        switch (moveToAttack.getType()){
            case 1://propose rule
                if(moveToAttack.getSource()!= playerIndx){
                    strategy.setDefendingStatus(false);
                    move = this.playUnwnatedConsequences(moveToAttack, strategy, playerIndx);
                    if (move == null){
                        move = this.playDistinguish(moveToAttack, strategy, playerIndx);
                        if (move != null)//distinguish OK
                            this.insertRuleintoRulelist(move.getRule());
                    }
                    else//unwanted ok
                        this.insertRuleintoRulelist(move.getRule());
                }
                break;
            case 2://Distunguish
                if(moveToAttack.getTarget()== playerIndx){
                    move = this.playUnwnatedConsequences(moveToAttack, strategy, playerIndx);
                    if (move != null)//unwanted succeeded
                        this.insertRuleintoRulelist(move.getRule());
                }
                break;
            case 3://unwanted
                if(moveToAttack.getTarget()== playerIndx){
                    move = this.playWithdraw(moveToAttack, strategy, playerIndx);
                    if (move != null)//unwanted succeeded
                        this.insertRuleintoRulelist(move.getRule());                    
                }
                break;
            case 4://counter rule
                if(moveToAttack.getSource()!= playerIndx){
                    strategy.setDefendingStatus(false);
                    move = this.playUnwnatedConsequences(moveToAttack, strategy, playerIndx);
                    if (move == null){
                        move = this.playDistinguish(moveToAttack, strategy, playerIndx);
                        if (move != null)//distinguish OK
                            this.insertRuleintoRulelist(move.getRule());
                    }
                    else//unwanted ok
                        this.insertRuleintoRulelist(move.getRule());                    
                }
                break;
            case 5://increase conf
                if(moveToAttack.getSource()!= playerIndx){
                    move = this.playUnwnatedConsequences(moveToAttack, strategy, playerIndx);
                    if (move == null){
                        move = this.playDistinguish(moveToAttack, strategy, playerIndx);
                        if (move != null)//distinguish OK
                            this.insertRuleintoRulelist(move.getRule());
                    }
                    else//unwanted ok
                        this.insertRuleintoRulelist(move.getRule());                      
                }
                break;
            case 6://withdraw
                if(moveToAttack.getSource()!= playerIndx){
                    move = this.playUnwnatedConsequences(moveToAttack, strategy, playerIndx);
                    if (move == null){
                        move = this.playDistinguish(moveToAttack, strategy, playerIndx);
                        if (move != null)//distinguish OK
                            this.insertRuleintoRulelist(move.getRule());
                    }
                    else//unwanted ok
                        this.insertRuleintoRulelist(move.getRule());                      
                }
                break;                
        }
        return move;
    }
   /*-------------------------------------------------------------------------------*/ 
    /*------------PLAY MOVES-------------*/   
    protected Move proposeFirstMove(Strategy strategy){
          RuleList ruleList = this.proposeRule(strategy.getUpThreshold(),this.goal, strategy);
	  if ((ruleList==null)||(ruleList.getNumCRs() ==0))
             return null;
	  else
	  {
              //strategy.setDefendingStatus(true);
              /*choose best move in the list according to strategy*/
              RuleNode rule = ruleList.cutFirstRule();
              Move move =  new Move(1, 0,1,rule,-1);
              return move;
          }      
    }        
    private Move playPropose(Move moveToCounter,Strategy strategy, int playerIndx){
          RuleList ruleList = this.proposeRule(moveToCounter.getRule().getConfidence(),this.goal, strategy);	  
	  if ((ruleList==null)||(ruleList.getNumCRs() ==0))
	    return null;	  
	  else{
              //strategy.setDefendingStatus(true);
              /*choose best move in the list according to strategy*/
              RuleNode rule = ruleList.cutFirstRule();
              Move move = new Move(playerIndx, moveToCounter.getSource(),1,rule, moveToCounter.getLeafIndx());
              return move;
          }      
    }
    protected Move playProposeWithHigherConf(Move moveToCounter, double confToCounter,Strategy strategy, int playerIndx){
          RuleList ruleList = this.proposeRule(confToCounter,this.goal,strategy);	  
	  if ((ruleList==null)||(ruleList.getNumCRs() ==0))
	    return null;
	  else{
              //strategy.setDefendingStatus(true);
              /*choose best move in the list according to strategy*/
              RuleNode rule = ruleList.cutFirstRule();
              Move move = new Move(playerIndx, moveToCounter.getSource(),1,rule, moveToCounter.getLeafIndx());
              return move;
          }      
    }
    private Move playCounterRule(Move moveToCounter,Strategy strategy, int playerIndx){
	  RuleNode ruleToCounter = moveToCounter.getRule();
	  double confidence= ruleToCounter.getConfidence();
	  RuleList ruleList = this.proposeRule(confidence, this.goal, strategy);
	  if ((ruleList==null)||(ruleList.getNumCRs() ==0)){
	    //System.out.println("Counter Rule fails --> No rules generated!");
	    return null;
	  }
	  else{
	    //strategy.setDefendingStatus(true);
	    /*choose best move in the list according to strategy*/
	    RuleNode rule = ruleList.cutFirstRule();	     
	    Move move = new Move(playerIndx, moveToCounter.getSource(),4, rule, moveToCounter.getLeafIndx());
	    return move;
	  }	      
    }   
    private Move playUnwnatedConsequences(Move moveToAttack,Strategy strategy, int playerIndx){
	RuleNode ruleToAttack = moveToAttack.getRule();
	boolean unWanted = this.unwantedConsequences(ruleToAttack);
	if (unWanted){
	  Move move = new Move(playerIndx,moveToAttack.getSource() ,3, ruleToAttack, moveToAttack.getLeafIndx());
	  return move;	  
	}
	else    return null;
    }    
    private Move playDistinguish(Move moveToDistinguish,Strategy strategy, int playerIndx){
	RuleNode ruleToDistinguish = moveToDistinguish.getRule();
	RuleList ruleList = distinguishDepthFirst(ruleToDistinguish,strategy);
        if ((ruleList==null)||(ruleList.getNumCRs() ==0))	  
	  return null;	
	else{
	  /*choose first move in the list -- to be replaced with content strategy*/
	  RuleNode rule = ruleList.getLastRule();
	  Move move = new Move(playerIndx, moveToDistinguish.getSource(),2, rule, moveToDistinguish.getLeafIndx());
	  return move;
	}
    }
    private Move playIncreaseConfidence(Move moveToIncrease, Strategy strategy, int playerIndx){
	RuleNode ruleToIncrease = moveToIncrease.getRule();
	RuleList ruleList = this.increaseConfidenceDepthFirst(ruleToIncrease, strategy);	
        if ((ruleList==null)||(ruleList.getNumCRs() ==0))
	  return null;
	else
	{
	  RuleNode rule = ruleList.getFirstRule();
	  Move move = new Move(playerIndx, moveToIncrease.getSource(),5, rule, moveToIncrease.getLeafIndx());        
	  return move;
	}		
    }
    private Move playWithdraw(Move moveToFix, Strategy strategy, int playerIndx){
	RuleNode ruletoFix = moveToFix.getRule();
	RuleList ruleList = this.withdrawUnwantedConsequences(ruletoFix,strategy);
        if ((ruleList==null)||(ruleList.getNumCRs() ==0))   return null;
	else{
	  /*choose first move in the list -- to be replaced with content strategy*/
	  RuleNode rule = ruleList.getFirstRule();
	  Move move = new Move(playerIndx, moveToFix.getSource(),6, rule, moveToFix.getLeafIndx());       
	  return move;
	}		
    }
    /*-------------------------------------------------------------------------------*/
    
    //GAME MOVES 
    /*------------PROPOSE RULE-------------*/
    /**returns a new rule (propose rule) or null if such rule couldnt be found*/
    private RuleList proposeRule(double confidence, short goal, Strategy strategy){
        RuleList ruleList = new RuleList();
	ruleList = proposeRulelevel1(confidence, goal,strategy);
        RuleNode firstRule = ruleList.getFirstRule();
	
	if((firstRule==null)||(firstRule.getConfidence()<strategy.getUpThreshold())){
	  short level = 2;
          boolean done =false;
	  while ((!done)&&(level<=currentCase.length)&&(level<strategy.getMaxLevel())){
	    RuleList newList = proposeRulelevelN(confidence, level, goal,strategy);
	    ruleList.merge(newList);
            firstRule = ruleList.getFirstRule();
            if((firstRule!=null)&&(firstRule.getConfidence()>=strategy.getUpThreshold()))
             done = true;
	    else
	     level++;
	  }
	}  		
	return ruleList;
    }
        /**return the first best rule attribute-->goal*/
    private RuleList proposeRulelevel1(double confidence,short goal, Strategy strategy){
	  double minConf = strategy.getMinConf();
	  if(minConf<confidence){minConf = confidence;}
          RuleList ruleList = new RuleList();
	  int i = 0;
	  boolean done = false;
	  while ((i<currentCase.length)&&(!done)){
             short[] level1PossibleFrequentSet = new short[2];
             level1PossibleFrequentSet[0] = currentCase[i];
             level1PossibleFrequentSet[1] = goal;
             //get the support of this set
             double support = pTree.getSupportForItemSetInTtree(level1PossibleFrequentSet);	
             if(support>0){
               //produce the rule List of the given set
	       short[] restList = new short[1];
	       restList[0] = level1PossibleFrequentSet[0];
	       short[] classList = new short[1];
	       classList[0] = level1PossibleFrequentSet[1];
	       ruleList = pTree.generateClassARsFromItemsetConfStrategy
			  (restList, classList, support, 
                           minConf, strategy.getMaxConf(),
                           this.playedRules);
               RuleNode firstRule = ruleList.getFirstRule();
	       if((firstRule!=null)&&(firstRule.getConfidence() >confidence))
                done=true;
	       else
	        i++;
             }
             else
	      i++;	   	
	}
        return ruleList;
    }    
    /**return the first best rule A-->goal : A is a level-subset of current Case*/
    private RuleList proposeRulelevelN(double confidence, short level, short goal,Strategy strategy){
	  double minConf = strategy.getMinConf();
	  if(minConf<confidence)    minConf = confidence;
	  boolean done = false;
          RuleList ruleList = new RuleList();
	  //generate levelwise subsets one by one
	  short caseSize = (short) currentCase.length;
	  SubsetsGenerator generator = new SubsetsGenerator(caseSize, level);
	  while ((generator.HasNext())&&(!done)){
	    short[] indexSet = generator.Next();
	    short[] itemSubset = new short [indexSet.length+1];
	    for(int i =0; i<indexSet.length; i++)	    
		itemSubset[i] = currentCase[indexSet[i]];            	    
            itemSubset[indexSet.length] = goal;
	    //for each generated l-subset check for support
            Arrays.sort(itemSubset);
	     double support = pTree.getSupportForItemSetInTtree(itemSubset);
	     //if support > min --> produce rule and check for confidence
             if(support>0){
	       short[] restList = new short[indexSet.length];
	       for(int i =0; i<indexSet.length; i++)	    	
                   restList[i] = currentCase[indexSet[i]];            	
               Arrays.sort(restList);
	       short[] classList = new short[1];
	       classList[0] = goal;
	       ruleList = pTree.generateClassARsFromItemsetConfStrategy
			  (restList, classList, support, 
                           minConf, strategy.getMaxConf(),
                           this.playedRules);
	       RuleNode rule =  ruleList.getFirstRule();
	       if((rule!=null)&&(rule.getConfidence() >confidence)
                              &&(rule.getConfidence() >=strategy.getUpThreshold()))
                done=true;
	     }
	  }
        return ruleList;
    }
    /*-------------------------------------------------------------------------------*/
        /*--------UNWANTED CONSEQUENCE----------*/
    /**returns true if the rule has any unwanted consequences */
    private boolean unwantedConsequences(RuleNode ruleToAttack){
	short [] consequents = ruleToAttack.getConsequent();        
	int i=0; 
	boolean consequencesOK = true;
        Arrays.sort(this.currentCase);
        Arrays.sort(this.counterGoals);       
	while((i<consequents.length) && (consequencesOK))
	{
            if((consequents[i]==this.goal)||
               (Arrays.binarySearch(this.counterGoals, consequents[i])>=0)||
               (Arrays.binarySearch(this.currentCase, consequents[i])>=0)){
                i++;
            }
            else
              consequencesOK = false;  
	}
	return !consequencesOK;
    }
    private short[] determineUnwantedConsequences(short [] consequents){
     short[] unwantedList = null;
     Arrays.sort(this.currentCase);
     Arrays.sort(this.counterGoals);
     for (int i =0; i< consequents.length; i++){
         if ((consequents[i]!=this.goal)&&
             (Arrays.binarySearch(this.currentCase,consequents[i])<0)&&
             (Arrays.binarySearch(this.counterGoals,consequents[i])<0)){
             short [] unwantedCons = new short [1];
             unwantedCons[0] = consequents[i];             
             unwantedList = BasicOperations.append(unwantedList, unwantedCons);
         }
     }   
     return unwantedList;
   }
    /**returns the set of unwanted consequences*/
    private short[] determineWantedConsequences(short [] consequents){
     Arrays.sort(this.currentCase);
     Arrays.sort(this.counterGoals);
     short[] wantedList = null;
     for (int i =0; i< consequents.length; i++){
         if ((consequents[i]==this.goal)||
             (Arrays.binarySearch(this.currentCase,consequents[i])>=0)||
             (Arrays.binarySearch(this.counterGoals,consequents[i])>=0)){
             short [] wantedCons = new short [1];
             wantedCons[0] = consequents[i];             
             wantedList= BasicOperations.append(wantedList, wantedCons);
         }
     }   
     return wantedList;
   }
    /*-------------------------------------------------------------------------------*/

    /*------------DISTINGUISH----------------*/
    /**delete the case attributes used in the input rule*/
    private short[] subRuleOfCase(short[] inputCase, short[] antecednt, short[]consequent){     
     short[] newCase = null;
     Arrays.sort(antecednt);
     Arrays.sort(consequent);
     for(int i =0; i<inputCase.length; i++)
     {         
         if((Arrays.binarySearch(antecednt,inputCase[i])<0)&&
            (Arrays.binarySearch(consequent,inputCase[i])<0)){
             short[] newElem = new short[1];
             newElem[0] =inputCase[i]; 
             newCase = BasicOperations.append(newCase, newElem);
         }
     }   
     return newCase;
   }
   /**return the first best rule A+B-->C 
       which confidence is lower than the original rule (A-->C)*/
    private RuleList distinguishDepthFirst(RuleNode inputRule, Strategy strategy){
	//produce the set to be porcessed
	short[] antecednt = inputRule.getAntecednt();
	short[] consequent = inputRule.getConsequent();
	//double confidence = inputRule.GetConfidence();
	double distConf = strategy.getDistThreshold();
	double minConf = strategy.getMinConf();
	//delete the case attributes used in the input rule 
	short[] newCase = this.subRuleOfCase(currentCase,antecednt, consequent); 
        if(newCase!= null){
          RuleList ruleList = distinguishDepthFirstlevel1
		   (strategy.getMaxLevel(), newCase, 
		    antecednt, consequent, minConf, distConf);
          return ruleList;
        }
        else
        return null;
    }
    private RuleList distinguishDepthFirstlevel1
	(int maxLevel,short[] newCase, 
	 short[] antecednt, short[] consequent, 
	 double minConfidence, double disConfidence){
          RuleList ruleList = new RuleList();
	  short i = 0;
	  boolean done = false;
	  while ((i<newCase.length)&&(!done)){
	     //expand the  antecedent with the current element of newCase
             short[] currentItemSet = new short [1];
	     currentItemSet[0] = newCase[i];             
	     short[] expAntecedent = BasicOperations.append(antecednt, currentItemSet);
 	     Arrays.sort(expAntecedent);
	     //merge both expanded antecednt and consequent
	     short expItemSet[] = BasicOperations.append(expAntecedent, consequent);
	     
	     Arrays.sort(expItemSet);
             //get the support of this set
	     double support=0;
	     if(expItemSet.length <=maxLevel)
                support = pTree.getSupportForItemSetInTtree(expItemSet);	
	     if(support>0){ 	
	       //check confidence of  new rule
	       double ruleConf = pTree.getConfidence(expAntecedent, support);
	       if(ruleConf< minConfidence){
		   RuleNode rule = new RuleNode (expAntecedent,
		     			   consequent,disConfidence);
		    boolean found;
                    if (this.playedRules == null)
                        found = false;
                    else
                    found = this.playedRules.existsInThisList(rule);
		    if(!found){
		      ruleList.insertRuleintoRulelist(expAntecedent,
		     				   consequent,disConfidence);
                      if(ruleConf<= disConfidence)
		         done=true;
		      else
		        i++;
		    }
                    else {//the rule already exists --> search subtree of this rule???
                        i++;
		  }
	       }
	       else {//confidence
	       //check super itemsets of this node
		  ruleList = pTree.distinguishConclusionHistory
			     (expAntecedent, consequent, 
                              minConfidence, disConfidence,this.playedRules);
		  RuleNode lastRule = ruleList.getLastRule();
		  if((lastRule!=null)&&(lastRule.getConfidence() <= disConfidence)) 
                       done=true;
		  else i++;
		}
             }
             else //support
	      i++;	   	
	}
        return ruleList;
    }
    /*-------------------------------------------------------------------------------*/
    /*------------INCREASE CONFIDENCE----------------*/
    /**return the first best rule A+B-->C 
       which confidence is higher than the original rule (A-->C)*/
    private RuleList increaseConfidenceDepthFirst(RuleNode inputRule, Strategy strategy){
	//produce the set to be porcessed
	short[] antecednt = inputRule.getAntecednt();
	short[] consequent = inputRule.getConsequent();
	double confidence = inputRule.getConfidence();
	double minConf = strategy.getMinConf();
	if(confidence<minConf)
	  confidence = minConf;
	//delete the case attributes used in the input rule 
	short[] newCase = subRuleOfCase(currentCase,antecednt, consequent);
	RuleList ruleList = increaseConfidenceDepthFirstLevel1
		   (strategy.getMaxLevel(), newCase, 
		    antecednt, consequent, 
		    confidence, strategy.getUpThreshold());
	return ruleList;
    }
    private RuleList increaseConfidenceDepthFirstLevel1
	(int maxLevel,short[] newCase, short[] antecednt, short[] consequent, 
	   double minConfidence, double upThreshold){
        RuleList ruleList = new RuleList();
	short i = 0;
	boolean done = false;
	while ((newCase!=null)&&(i<newCase.length)&&(!done)){
	     //expand the  antecedent with the current element of newCase
             short[] currentItemSet = new short [1];
	     currentItemSet[0] = newCase[i];
	     short[] expAntecedent = BasicOperations.append(antecednt, currentItemSet);
 	     Arrays.sort(expAntecedent);
	     //merge both expanded antecednt and consequent
	     short expItemSet[] = BasicOperations.append(expAntecedent, consequent);
	     
	     Arrays.sort(expItemSet);
             //get the support of this set
	     double support;
	     if(expItemSet.length <=maxLevel)
             support = pTree.getSupportForItemSetInTtree(expItemSet);	
	     else
	     support = 0;
	     if(support>0){
		//check confidence of  new rule
	        double confidence = pTree.getConfidence(expAntecedent, support);
		if(confidence> minConfidence){
		  RuleNode rule = new RuleNode (expAntecedent,
		    			   consequent,confidence);
                  boolean found;
                  if (this.playedRules == null)
                        found = false;
                  else
                    found = this.playedRules.existsInThisList(rule);
		  if(!found){
		    ruleList.insertRuleintoRulelist(expAntecedent,
		    				   consequent,confidence);
		    if(confidence>= upThreshold)
		     done=true;
		    else i++;
		  }
		  else //the rule already exists --> search subtree of this rule???
                      i++;		  
		}
		else{ // confidence
		    //check super itemsets of this node
		    ruleList = pTree.increaseConfidenceConclusionHistory
			     (expAntecedent, consequent, 
			      minConfidence,upThreshold,
			      this.playedRules);
		  RuleNode firstRule = ruleList.getFirstRule();
		  if((firstRule!=null)&&(firstRule.getConfidence() >=upThreshold)) 
                       done=true;
		    else i++;
		}
	     }
	     else i++; //support
	}
	return ruleList;
    }
    /*-------------------------------------------------------------------------------*/
    /*------------WITHDRAW UNWANTED CONSEQUENCES----------------*/
    /**Input rule: A-->X+Y (such that Y is unwanted cosequnecses)
     * return the first best rule A+B-->X+Z
     * which confidence is at least equal to the strategy minimun cofidence, and such that it X+Z has no
     * unwanted consequences*/
    private RuleList withdrawUnwantedConsequences
		   (RuleNode inputRule, Strategy strategy){
	RuleList ruleList = new RuleList();
	//produce the set to be porcessed
	short[] antecednt = inputRule.getAntecednt();
	short[] consequent = inputRule.getConsequent();
	double confidence = inputRule.getConfidence();
	//determine the unwanted consequences
	short[] wantedConsequneces = this.determineWantedConsequences(consequent);
	short[] newItemSet = BasicOperations.append(antecednt,wantedConsequneces);
	Arrays.sort(newItemSet);
	double support = pTree.getSupportForItemSetInTtree(newItemSet);
	if(support>0){
	   //get confidence
	   double withConfidence = pTree.getConfidence(antecednt, support);
	   if (strategy.getDefendingStatus()){
		if(withConfidence>=strategy.getMinConf()){		
 		  RuleNode rule = new RuleNode (antecednt,
		    			   wantedConsequneces,withConfidence);
		  boolean found;
                  if (this.playedRules == null)
                        found = false;
                    else
                    found = this.playedRules.existsInThisList(rule);
		  if(!found){
		    ruleList.insertRuleintoRulelist(antecednt,
		    			   wantedConsequneces,withConfidence);
		  }
	   	}
	   }
	   else{
               if(withConfidence<strategy.getDistThreshold()){		
		   RuleNode rule = new RuleNode (antecednt,
		    			   wantedConsequneces,withConfidence);
		    boolean found;
                    if (this.playedRules == null)
                        found = false;
                    else
                        found = this.playedRules.existsInThisList(rule);
		    if(!found)
		      ruleList.insertRuleintoRulelist(antecednt,
		    			   wantedConsequneces,withConfidence);		    
	   	}
	   }
	}
	return ruleList;	
    }
    /*-------------------------------------------------------------------------------*/
}
