/*
 * Strategy.java
 * Encapuslates PISA players strategy parameters and functions
 * Created on 02 September 2008, 16:15
 *
 *@author maya
 */


package GUIComponents;
//java packages
import java.io.*;
import java.util.*;

public class Strategy {  
     private int gameMode = StrategyConstants.GAME_MODE_WIN; //1 = win, 2 = game
     private int agentProfile = StrategyConstants.AGENT_PROFILE_DISAGREEABLE; //1 = disagreeable, 2 = agreeable
     private int strategyMode = StrategyConstants.STRATEGY_MODE_BUILD; //1 = build, 2 = destroy
     private int strategyType=StrategyConstants.ATTACK_WHEN_POSSIBLE; //1 attack whenever possible, 2 attack when needed, 3 attack to prevent a threat
     private int smartness=StrategyConstants.SMARTNESS_BLIND;/*0 Blind (default)  - 1 Focused - 2 Tree Oriented*/
     private int experience=0; //the number of rows in the player's data set
     private double min_conf = 30.0;//the minimum confidence threshold for a rule to be accepted
     private double max_conf = 99.0;//the maximum confidence threshold for a rule to be accepted
     private double dist_threshold=20.0;//maximum confidence for a distinguished rule to be accepted
     /*max confidence for a rule of a given level in a drop down mode to be accepted*/
     private double drop_threshold = 20.0;
     /*max confidence for a rule of a given level in an upward mode to be accepted*/
     private double up_threshold=50.0;
     private int max_level;//maximum number of levels to be considered
     private int considered_levels_no=6;//maximum increase in levels to be considered
     /*indicates whether the player is in defending or attacking mode*/
     private boolean defending=true; 
      
     /**Default Constructor*/
     public Strategy(){}
       
     /* ------ SETTERS&GETTERS ------ */
   
     protected int getGameMode (){return this.gameMode;}
     protected void setGameMode (int game){
         if(StrategyConstants.checkGameMode(game))
             this.gameMode = game;         
     }
     protected int getAgentProfile (){return this.agentProfile;}
     protected void setAgentProfile (int profile){
         if(StrategyConstants.checkAgentProfile(profile))
             this.agentProfile = profile;
     }
     protected int getStrategyMode (){return this.strategyMode;}
     protected void setStrategyMode (int mode){
         if(StrategyConstants.checkStrategyMode(mode))
             this.strategyMode = mode;
     }
     
     protected int getExperience (){return this.experience;}
     protected void setExperience  (int newExperience){this.experience = newExperience;}

     protected double getMinConf (){return this.min_conf;}
     protected void setMinConf (double minConf){ this.min_conf = minConf;}

     protected double getMaxConf (){return this.max_conf;}
     protected void setMaxConf (double maxConf){this.max_conf = maxConf;}
     
     protected double getDistThreshold (){return this.dist_threshold;}
     protected void setDistThreshold (double distThreshold){this.dist_threshold = distThreshold;}

     protected double getDropThreshold (){return this.drop_threshold;}
     protected void setDropThreshold (double dropThreshold){this.drop_threshold = dropThreshold;}

     protected double getUpThreshold (){return this.up_threshold;}   
     protected void setUpThreshold (double upThreshold){this.up_threshold = upThreshold;}
   
     protected int getMaxLevel(){return this.max_level;}  
     protected void setMaxLevel (int maxLevel){this.max_level = maxLevel;}

     protected int getConsideredLevels (){return this.considered_levels_no;}
     protected void setConsideredLevels (int levelIncrease){this.considered_levels_no = levelIncrease;}
  
     protected boolean getDefendingStatus (){return this.defending;}
     protected void setDefendingStatus (boolean defendingStatus){ this.defending = defendingStatus;}
     
     protected int getSmartness(){return this.smartness;}
     protected void setSmartness(int newSmartnessLevel){
         if(StrategyConstants.checkSmartness(newSmartnessLevel))
             this.smartness = newSmartnessLevel;
     }
   
     protected int getStrategyType(){return this.strategyType;}
     protected void setStrategyType(int newStrategyType){
         if(StrategyConstants.checkStrategyType(newStrategyType))
             this.strategyType = newStrategyType;
     }
   
     /**set strategy in Attack When Possible mode*/
     protected void setAttackWhenPossible(int mode){
       this.strategyMode = mode;
       this.strategyType = 1;   
     }
     /**set strategy in Attack When Needed mode*/   
     protected void setAttackWhenNeeded(int mode){
       this.strategyMode = mode;
       this.strategyType = 2;
     }   
     /**set strategy in Attack To Prevent Possible Threat mode*/
     protected void setAttackToPrevent(int mode){
       this.strategyMode = mode;
       this.strategyType = 3;
     }      
     
     /**output the strategy TYPE*/
     protected String outputType(){
       switch(this.strategyType){
           case StrategyConstants.ATTACK_WHEN_POSSIBLE:
               return "Strategy Type: attack whenever possible";
           case StrategyConstants.ATTACK_WHEN_NEEDED:
               return "Strategy Type: attack when needed";
           case StrategyConstants.ATTACK_TO_PREVENT_THREAT:
               return "Strategy Type:attack to prevent threat";
           default: return null;
       }
   }
     /**output the strategy Mode*/
     protected String outputMode(){
       switch(this.strategyMode){
           case StrategyConstants.STRATEGY_MODE_BUILD:
               return "Strategy Mode: Build";               
           case StrategyConstants.STRATEGY_MODE_DESTROY:
               return "Strategy Mode: Destroy";
           default: return null;
       }
   }
     /**output the strategy Smartness*/
     protected String outputSmartness(){
       switch(this.smartness){
           case StrategyConstants.SMARTNESS_BLIND:
               return "Smartness Level: Blind";
           case StrategyConstants.SMARTNESS_FOCUSED:
               return"Smartness Level: Focused";               
           case StrategyConstants.SMARTNESS_TREE_ORIENTED:
               return "Smartness Level: Tree Oriented";
           default: return null;
       }
    }  
     /**output the strategy*/
     protected void output(){
           System.out.println(this.outputType());
           System.out.println(this.outputMode());
           System.out.println(this.outputSmartness());
       }
     
     protected String outputParameters(){
         String output = this.outputType() +"\n";
         output+=this.outputMode()+"\n";
         output+=this.outputSmartness()+"\n";
         return output;
       }
   
   /*the ranking of strategies is as follows:
    1   - Prevent Threat + Build
    2   - Prevent Threat + Destroy
    3   - Attack when needed + Tree + Build
    4   - Attack when needed + Tree + Destroy
    5   - Attack when possible + Tree + Build
    6   - Attack when possible + Tree + Destroy
    7   - Attack when needed Focused + Build
    8   - Attack when needed Focused + Destroy
    9   - Attack when possible Focused + Build
    10  - Attack when possible Focused + Destroy
    11  - Attack when needed Blind + Build
    12  - Attack when needed Blind + Destroy
    13  - Attack when possible Blind + Build
    14  - Attack when possible Blind + Destroy*/   
     protected int strategyRank(){
       switch(this.strategyType){
           case StrategyConstants.ATTACK_TO_PREVENT_THREAT://prevent Threat
               if(this.strategyMode==StrategyConstants.STRATEGY_MODE_BUILD) return 1;//build
               else return 2;//destroy               
           case StrategyConstants.ATTACK_WHEN_NEEDED://needed
               if(this.smartness==StrategyConstants.SMARTNESS_TREE_ORIENTED){//Tree Oriented
                   if(this.strategyMode==StrategyConstants.STRATEGY_MODE_BUILD) return 3;//build
                   else return 4;//destroy               
               }
               else{
                   if(this.smartness==StrategyConstants.SMARTNESS_FOCUSED){//Focused
                       if(this.strategyMode==StrategyConstants.STRATEGY_MODE_BUILD) return 7;//build
                       else return 8;//destroy        
                   }
                   else{//Blind
                       if(this.strategyMode==StrategyConstants.STRATEGY_MODE_BUILD) return 11;//build
                       else return 12;//destroy    
                   }
               }               
           case StrategyConstants.ATTACK_WHEN_POSSIBLE://possible
               if(this.smartness==StrategyConstants.SMARTNESS_TREE_ORIENTED){//Tree Oriented
                   if(this.strategyMode==StrategyConstants.STRATEGY_MODE_BUILD) return 5;//build
                   else return 6;//destroy               
               }
               else{
                   if(this.smartness==StrategyConstants.SMARTNESS_FOCUSED){//Focused
                       if(this.strategyMode==StrategyConstants.STRATEGY_MODE_BUILD) return 9;//build
                       else return 10;//destroy        
                   }
                   else{//Blind
                       if(this.strategyMode==StrategyConstants.STRATEGY_MODE_BUILD) return 13;//build
                       else return 14;//destroy    
                   }
               }                              
       }
       return -1;//error value
   }
}//end of the class

