/*
 * TMU.java
 *
 * Created on 03 September 2008, 12:06
 *
 * Tree Management Unit class: responsible for
 * 1) adding new moves to the argumentation tree
 * 2) prunning player's p nodes from argumentation tree 
 * 3) coloring the tree after each round
 * 4) prviding the basic tree infromation needed by the players
 * @author maya
 */

package GUIComponents;

public class TMU {
    ArgTree argTree; //the argumentation tree
    
    /**TMU constructor*/
    public TMU(int currentNoPlayers, int maxTreeSize) {
        argTree = new ArgTree(currentNoPlayers, maxTreeSize);
    }
    
    //methods
    /**adds round moves to the argumentation tree*/
    public boolean addRoundMoves (Move[] roundMoves, int round){
        return this.argTree.addRoundMoves(roundMoves,round);
    }
    /**adds the round best move to the argumentation tree*/
    public boolean addRoundBestMove (Move[] roundMoves, int round){
        FocusingUnit focusingUnit = new FocusingUnit(this.argTree);
        return focusingUnit.addRoundBestMove(roundMoves,round);
    }
    /**adds first round move to the argumentation tree*/
    public boolean addFirstMove(Move firstMove){
        this.argTree.addRootMove(firstMove);
        return true;
    }
    /**prunes all nodes played by the input player*/
    public void prunePlayer(int playerIndx){
        this.argTree.prune(playerIndx);        
    }
    /**prunes all nodes played by the input players*/
    public void prunePlayers(int[] playerIndxArray){
        for(int i=0; i<playerIndxArray.length; i++)
            this.prunePlayer(playerIndxArray[i]);
    }
    /**returns the tree leafs*/
    public ArgTreeNode[] requestLeafs(){
        return this.argTree.getLeafs();
    }    
    /**returns the tree leafs in compressed format*/
    public CompressedArgTreeNode[] requestCompressedLeafs(){        
        return this.argTree.getCompressedLeafs(0);
    }
    /**initialize the unit*/      
    public void initialize(int currentNoPlayers, int maxTreeSize){
        argTree = new ArgTree(currentNoPlayers, maxTreeSize);
    }
    /**returns read interface to the unit - focusing level = 1*/
    public TMUInterface createInterface(){
        return new TMUInterface(this.argTree);
    }
    /**returns read interface to the unit*/
    public TMUInterface createInterface(int focusingLevel){
        return new TMUInterface(this.argTree,focusingLevel);
    }
    /**announces who won the game*/
    public int[] announceWinners(){
        WinnerAnnouncementUnit wau = new WinnerAnnouncementUnit(this.argTree);       
        return wau.winnersList();
    }
}

