/*
 * WinnerAnnouncementUnit.java
 *
 * Created on 03 September 2008, 13:12
 *
 * announces the winner(s) at the end of the game, by looking up in the ARGTREE
 * @author maya
 */

package GUIComponents;

public class WinnerAnnouncementUnit {
    ArgTree argTree;
    /** Creates a new instance of WinnerAnnouncementUnit */
    public WinnerAnnouncementUnit(ArgTree finalArgTree) {
        this.argTree = finalArgTree;
    }
    
    private ArgTreeNode[] winningNodes(){
        ArgTreeNode[] winningNodes=null;
        if(!this.argTree.rootAttacked()){
            winningNodes = BasicOperations.append(winningNodes, argTree.copyRoot());
            return winningNodes;
        }
        if(this.argTree.hasGreenLeafs()){
            //Green WINNER or Green TIE
            ArgTreeNode[] leafs = this.argTree.getLeafs();
            for(int i=0; i<leafs.length; i++){
                if(leafs[i].getColour()==Colours.GREEN)
                    winningNodes = BasicOperations.append(winningNodes, leafs[i]);
            }
        }
        else{
            int blueDominant = this.argTree.dominantExclusiveBlue();
            if((blueDominant)>0){
                ArgTreeNode[] leafs = this.argTree.getLeafs();
                for(int i=0; i<leafs.length; i++){
                    if(leafs[i].getPlayer()==blueDominant)
                        winningNodes = BasicOperations.append(winningNodes, leafs[i]);
                }
            }            
        }
        return winningNodes;
    }
    /**return the winning nodes in the compressed form*/
    protected CompressedArgTreeNode[] compressedWinningNodes(){
        ArgTreeNode[] winningNodes = this.winningNodes();
        return argTree.getCompressedLeafs(0, winningNodes);
    }
    /**return true if the player is in the winners list*/
    private boolean winnerPlayerFound(int[] winners, int playerIndex){
        for(int i=0; i<winners.length; i++){
            if(winners[i] == playerIndex) return true;
        }
        return false;
    }
    /**return a list of the winners of the game (if any)*/
    protected int[] winnersList(){
        ArgTreeNode[] winningNodes = this.winningNodes();        
        if(winningNodes == null) return null;
        int[] winners = null;
        if(winningNodes[0]==null) return null;
        winners = BasicOperations.append(winners, winningNodes[0].getPlayer());
        for(int i=1; i<winningNodes.length; i++){
            if(!this.winnerPlayerFound(winners, winningNodes[i].getPlayer())){
                winners = BasicOperations.append(winners, winningNodes[i].getPlayer());
            }
        }
        return winners;
    }
}
